@@ -1,6 +1,6 @@
 /**
  * File name: EnvelopePointContextWidget.cpp
- * Project: Geonkick (A percussion synthesizer)
+ * Project: Geonkick (A percussive synthesizer)
  *
  * Copyright (C) 2023 Iurie Nistor
  *
@@ -24,8 +24,8 @@
 #include "EnvelopePointContextWidget.h"
 #include "envelope.h"
 
-#include <RkEvent.h>
-#include <RkLineEdit.h>
+#include "RkEvent.h"
+#include "RkLineEdit.h"
 
 EnvelopePointContextWidget::EnvelopePointContextWidget(Envelope* envelope,
                                                        GeonkickWidget *parent,
