@@ -1,8 +1,8 @@
 /**
  * File name: ExportToSfz.cpp
- * Project: Geonkick (A percussion synthesizer)
+ * Project: Geonkick (A percussive synthesizer)
  *
- * Copyright (C) 2020 Iurie Nistor 
+ * Copyright (C) 2020 Iurie Nistor
  *
  * This file is part of Geonkick.
  *
@@ -24,10 +24,10 @@
 #include "ExportToSfz.h"
 #include "ExportSoundData.h"
 #include "kit_model.h"
-#include "percussion_model.h"
+#include "InstrumentModel.h"
 
-ExportToSfz::ExportToSfz(KitModel *model, const std::filesystem::path &file)
-        : ExportAbstract(model, file, ExportFormat::Sfz)
+ExportToSfz::ExportToSfz(const KitModel *model, const std::filesystem::path &file)
+        : ExportAbstract(file, ExportFormat::Sfz)
         , kitModel{model}
 {
 }
@@ -35,13 +35,13 @@ ExportToSfz::ExportToSfz(KitModel *model, const std::filesystem::path &file)
 bool ExportToSfz::doExport()
 {
         std::ostringstream sfzStream;
-        sfzStream << "// Exported from " << Geonkick::appName
+        sfzStream << "// Exported from " << Geonkick::applicationName
             << " version " << Geonkick::applicationVersionStr << std::endl;
         sfzStream << "// Name: " << kitModel->name() << std::endl;
         sfzStream << "// Author: " << kitModel->author() << std::endl;
         sfzStream << "// License: " << kitModel->license() << std::endl;
         sfzStream << "// Instruments:" << std::endl;
-        for (const auto &per : kitModel->percussionModels())
+        for (const auto &per : kitModel->instrumentModels())
                 sfzStream << "//    * " << per->name() << std::endl;
         sfzStream << std::endl;
         sfzStream << "<control>" << std::endl;
@@ -67,7 +67,7 @@ bool ExportToSfz::doExport()
                 return false;
         }
 
-        for (const auto &instrument : kitModel->percussionModels()) {
+        for (const auto &instrument : kitModel->instrumentModels()) {
                 sfzStream << "<region>" << std::endl;
                 std::filesystem::path wavFile = dataPath()
                         / std::filesystem::path(cleanName(instrument->name()) + ".wav");
@@ -79,19 +79,16 @@ bool ExportToSfz::doExport()
                 sfzStream << "hivel=127" << std::endl;
                 sfzStream << "volume=15" << std::endl;
 		sfzStream << std::endl;
-                ExportSoundData exportToWav(this,
-                                            wavFile,
+                ExportSoundData exportToWav(wavFile,
                                             instrument->data(),
                                             ExportSoundData::ExportFormat::Wav);
-                exportToWav.setSubformat(ExportSoundData::Subformat::Wav32);
+                exportToWav.setBitDepth(32);
+                exportToWav.setSampleRate(kitModel->getDspProxy()->getSampleRate());
                 exportToWav.doExport();
-                if (exportToWav.isError())
-                        setError(exportToWav.errorString());
         }
 
         auto sfzFilePath = getExportPath();
-        if (!sfzFilePath.has_extension() && sfzFilePath.extension() != ".sfz"
-            && sfzFilePath.extension() != ".SFZ")
+        if (!sfzFilePath.has_extension() && Geonkick::toLower(sfzFilePath.extension().string()) != ".sfz")
                 sfzFilePath += ".sfz";
         std::ofstream sfzFile (sfzFilePath);
         if (!sfzFile.is_open()) {
