@@ -1,6 +1,6 @@
 /**
  * File name: ExportToSfz.h
- * Project: Geonkick (A percussion synthesizer)
+ * Project: Geonkick (A percussive synthesizer)
  *
  * Copyright (C) 2020 Iurie Nistor 
  *
@@ -30,16 +30,15 @@ class KitModel;
 
 class ExportToSfz : public ExportAbstract {
  public:
-        ExportToSfz(KitModel *model, const std::filesystem::path &file);
+        ExportToSfz(const KitModel *model, const std::filesystem::path &file);
         bool doExport() override;
-        
 protected:
         std::filesystem::path dataPath() const;
         static std::string cleanName(const std::string &name);
 
  private:
         std::string sfzFileName;
-        KitModel *kitModel;
+        const KitModel *kitModel;
 };
 
 #endif // GEONGKICK_EXPORT_TO_SFZ_H
