@@ -22,23 +22,33 @@
  */
 
 #include "GeonkickModel.h"
-#include "geonkick_api.h"
+#include "DspProxy.h"
 #include "kit_model.h"
 #include "preset_browser_model.h"
+#include "OscillatorModel.h"
 
-GeonkickModel::GeonkickModel(RkObject* parent, GeonkickApi *api)
+GeonkickModel::GeonkickModel(RkObject* parent, DspProxy *dsp)
         : RkObject(parent)
-        , geonkickApi{api}
-        , kitModel{new KitModel(this, geonkickApi)}
-        , presetModel{new PresetBrowserModel(this, geonkickApi)}
+        , dspProxy{dsp}
+        , kitModel{new KitModel(this)}
+        , presetModel{new PresetBrowserModel(this, dspProxy)}
 {
+        auto n = dspProxy->oscillatorsPerLayer();
+        for (decltype(n) i = 0; i < n; i++)
+                oscillatorModels.emplace_back(new OscillatorModel(this, static_cast<OscillatorModel::Type>(i)));
 }
 
-GeonkickApi* GeonkickModel::api() const
+DspProxy* GeonkickModel::getDspProxy() const
 {
-        return geonkickApi;
+        return dspProxy;
 }
 
+const std::vector<OscillatorModel*>& GeonkickModel::getOscillatorModels() const
+{
+        return oscillatorModels;
+}
+
+
 KitModel* GeonkickModel::getKitModel() const
 {
         return kitModel;
@@ -49,4 +59,25 @@ PresetBrowserModel* GeonkickModel::getPresetsModel() const
         return presetModel;
 }
 
+GeonkickModel::InstanceType GeonkickModel::instanceType() const
+{
+        return dspProxy->getInstanceType();
+}
 
+GeonkickModel::KitType GeonkickModel::kitType() const
+{
+#if defined(GEONKICK_SINGLE)
+        return KitType::SingleInstrument;
+#else
+        return KitType::MultiInstrument;
+#endif // GEONKICK_SINGLE
+}
+
+bool GeonkickModel::isBasic() const
+{
+#if defined(GEONKICK_BASIC_VERSION)
+        return true;
+#else
+        return false;
+#endif // GEONKICK_BASIC_VERSION
+}
