@@ -25,22 +25,33 @@
 #define GEONKICK_MODEL_H
 
 #include "globals.h"
+#include "DspProxy.h"
 
 #include "RkObject.h"
 
-class GeonkickApi;
 class KitModel;
 class PresetBrowserModel;
+class OscillatorModel;
 
 class GeonkickModel : public RkObject {
  public:
-        explicit GeonkickModel(RkObject* parent, GeonkickApi *api);
-        GeonkickApi* api() const;
+        enum class KitType: int {
+                SingleInstrument,
+                MultiInstrument,
+        };
+        using InstanceType = DspProxy::InstanceType;
+        explicit GeonkickModel(RkObject* parent, DspProxy *dsp);
+        DspProxy* getDspProxy() const;
         KitModel* getKitModel() const;
         PresetBrowserModel* getPresetsModel() const;
+        const std::vector<OscillatorModel*>& getOscillatorModels() const;
+        InstanceType instanceType() const;
+        KitType kitType() const;
+        bool isBasic() const;
 
  private:
-        GeonkickApi *geonkickApi;
+        DspProxy *dspProxy;
+        std::vector<OscillatorModel*> oscillatorModels;
         KitModel *kitModel;
         PresetBrowserModel* presetModel;
 };
