@@ -1,6 +1,6 @@
 /**
  * File name: MidiKeyWidget.h
- * Project: Geonkick (A kick synthesizer)
+ * Project: Geonkick (A percussive synthesizer)
  *
  * Copyright (C) 2020 Iurie Nistor
  *
@@ -25,9 +25,9 @@
 #define GEONKICK_MIDIKEY_WIDGET_H
 
 #include "geonkick_widget.h"
-#include "percussion_model.h"
+#include "InstrumentModel.h"
 
-#include <RkPainter.h>
+#include "RkPainter.h"
 
 class KeyCell {
   public:
@@ -93,9 +93,10 @@ class MidiKeyWidget: public GeonkickWidget
         void onUpdateKey(PercussionModel::KeyIndex key);
         void closeEvent(RkCloseEvent *event) override;
         void hoverEvent(RkHoverEvent *event) override;
+        std::string midiKeyToString(GeonkickTypes::MidiKey key);
 
  private:
-        PercussionModel *percussionModel;
+        PercussionModel *instrumentModel;
         RkSize cellSize;
         int widgetPadding;
         int midiRows;
