@@ -1,6 +1,6 @@
 /**
  * File name: TopBar.cpp
- * Project: Geonkick (A kick synthesizer)
+ * Project: Geonkick (A percussive synthesizer)
  *
  * Copyright (C) 2018 Iurie Nistor
  *
@@ -28,21 +28,19 @@
 #include "ViewState.h"
 #include "GeonkickModel.h"
 #include "kit_model.h"
-#include "percussion_model.h"
+#include "InstrumentModel.h"
 #include "MidiKeyWidget.h"
 #include "PresetNavigator.h"
+#include "SettingsWidget.h"
 
-#include <RkLabel.h>
-#include <RkButton.h>
-#include <RkContainer.h>
-#include <RkSpinBox.h>
+#include "RkLabel.h"
+#include "RkButton.h"
+#include "RkContainer.h"
+#include "RkSpinBox.h"
+#include "RkLineEdit.h"
 
 RK_DECLARE_IMAGE_RC(separator);
 RK_DECLARE_IMAGE_RC(logo);
-RK_DECLARE_IMAGE_RC(open);
-RK_DECLARE_IMAGE_RC(open_hover);
-RK_DECLARE_IMAGE_RC(save);
-RK_DECLARE_IMAGE_RC(save_hover);
 RK_DECLARE_IMAGE_RC(export);
 RK_DECLARE_IMAGE_RC(export_hover);
 RK_DECLARE_IMAGE_RC(reset);
@@ -51,7 +49,7 @@ RK_DECLARE_IMAGE_RC(reset_active);
 RK_DECLARE_IMAGE_RC(play);
 RK_DECLARE_IMAGE_RC(play_pressed);
 RK_DECLARE_IMAGE_RC(play_hover);
-#ifndef GEONKICK_LIMITED_VERSION
+#ifndef GEONKICK_BASIC_VERSION
 RK_DECLARE_IMAGE_RC(layer1);
 RK_DECLARE_IMAGE_RC(layer2);
 RK_DECLARE_IMAGE_RC(layer3);
@@ -61,7 +59,7 @@ RK_DECLARE_IMAGE_RC(layer3_disabled);
 RK_DECLARE_IMAGE_RC(layer1_hover);
 RK_DECLARE_IMAGE_RC(layer2_hover);
 RK_DECLARE_IMAGE_RC(layer3_hover);
-#endif // GEONKICK_LIMITED_VERSION
+#endif // GEONKICK_BASIC_VERSION
 RK_DECLARE_IMAGE_RC(tune_checkbox_on);
 RK_DECLARE_IMAGE_RC(tune_checkbox_off);
 RK_DECLARE_IMAGE_RC(tune_checkbox_hover);
@@ -71,21 +69,14 @@ RK_DECLARE_IMAGE_RC(topmenu_controls_off);
 RK_DECLARE_IMAGE_RC(topmenu_midi_off);
 RK_DECLARE_IMAGE_RC(topmenu_midi_active);
 RK_DECLARE_IMAGE_RC(topmenu_midi_hover);
-#ifdef GEONKICK_SINGLE
 RK_DECLARE_IMAGE_RC(note_off_unpressed);
 RK_DECLARE_IMAGE_RC(note_off_hover);
 RK_DECLARE_IMAGE_RC(note_off_pressed);
-#else
+#ifndef GEONKICK_SINGLE
 RK_DECLARE_IMAGE_RC(topmenu_kit_active);
 RK_DECLARE_IMAGE_RC(topmenu_kit_hover);
 RK_DECLARE_IMAGE_RC(topmenu_kit_off);
 #endif // GEONKICK_SINGLE
-RK_DECLARE_IMAGE_RC(topmenu_presets_active);
-RK_DECLARE_IMAGE_RC(topmenu_presets_hover);
-RK_DECLARE_IMAGE_RC(topmenu_presets_off);
-RK_DECLARE_IMAGE_RC(topmenu_samples_active);
-RK_DECLARE_IMAGE_RC(topmenu_samples_hover);
-RK_DECLARE_IMAGE_RC(topmenu_samples_off);
 RK_DECLARE_IMAGE_RC(topmenu_settings_active);
 RK_DECLARE_IMAGE_RC(topmenu_settings_hover);
 RK_DECLARE_IMAGE_RC(topmenu_settings_off);
@@ -93,82 +84,58 @@ RK_DECLARE_IMAGE_RC(topmenu_settings_off);
 TopBar::TopBar(GeonkickWidget *parent, GeonkickModel *model)
         : GeonkickWidget(parent)
         , geonkickModel{model}
-        , openFileButton{nullptr}
-        , saveFileButton{nullptr}
         , exportFileButton{nullptr}
         , presetNavigator{nullptr}
-#ifndef GEONKICK_LIMITED_VERSION
+#ifndef GEONKICK_BASIC_VERSION
         , layer1Button{nullptr}
         , layer2Button{nullptr}
         , layer3Button{nullptr}
-#endif // GEONKICK_LIMITED_VERSION
+#endif // GEONKICK_BASIC_VERSION
+        , instrumentName {nullptr}
         , controlsButton{nullptr}
         , midiKeyButton{nullptr}
-#ifdef GEONKICK_SINGLE
         , midiChannelSpinBox{nullptr}
         , noteOffButton{nullptr}
-#else
+#ifndef GEONKICK_SINGLE
         , kitButton{nullptr}
 #endif // GEONKICK_SINGLE
         , presetsButton{nullptr}
         , samplesButton{nullptr}
+        , settingsButton{nullptr}
 {
         setName("TopBar");
-        setFixedSize({parent->width(), 30});
+        setFixedSize({940 - 10, 30});
         auto mainLayout = new RkContainer(this);
         mainLayout->setSize(size());
 
         auto logo = new RkLabel(this);
         logo->setBackgroundColor(background());
         RkImage image(22, 22, RK_IMAGE_RC(logo));
+        image.grayscaleImage();
         logo->setSize(image.width(), image.height());
         logo->setImage(image);
         logo->show();
         mainLayout->addWidget(logo);
-        addSeparator(mainLayout, 5);
 
-        openFileButton = new GeonkickButton(this);
-        openFileButton->show();
-        openFileButton->setSize(26, 10);
-        openFileButton->setType(RkButton::ButtonType::ButtonCheckable);
-        openFileButton->setImage(RkImage(openFileButton->size(), RK_IMAGE_RC(open)),
+        // Setting button
+        addSeparator(mainLayout);
+        settingsButton = new GeonkickButton(this);
+        settingsButton->setType(RkButton::ButtonType::ButtonUncheckable);
+        settingsButton->setFixedSize(16, 16);
+        settingsButton->setImage(RkImage(settingsButton->size(),
+                                         RK_IMAGE_RC(topmenu_settings_off)),
                                  RkButton::State::Unpressed);
-        openFileButton->setImage(RkImage(openFileButton->size(), RK_IMAGE_RC(open_hover)),
-                                 RkButton::State::UnpressedHover);
-        openFileButton->setImage(RkImage(openFileButton->size(), RK_IMAGE_RC(open_hover)),
+        settingsButton->setImage(RkImage(settingsButton->size(),
+                                         RK_IMAGE_RC(topmenu_settings_active)),
                                  RkButton::State::Pressed);
-        RK_ACT_BINDL(openFileButton, pressed, RK_ACT_ARGS(), [=,this](){openFileButton->setPressed(false);
-                                                              action openFile();});
-        mainLayout->addWidget(openFileButton);
-        addSeparator(mainLayout, 5);
-
-        saveFileButton = new GeonkickButton(this);
-        saveFileButton->setSize(23, 10);
-        saveFileButton->setType(RkButton::ButtonType::ButtonCheckable);
-        saveFileButton->setImage(RkImage(saveFileButton->size(), RK_IMAGE_RC(save)),
-                                 RkButton::State::Unpressed);
-        saveFileButton->setImage(RkImage(saveFileButton->size(), RK_IMAGE_RC(save_hover)),
+        settingsButton->setImage(RkImage(settingsButton->size(),
+                                         RK_IMAGE_RC(topmenu_settings_hover)),
                                  RkButton::State::UnpressedHover);
-        saveFileButton->setImage(RkImage(saveFileButton->size(), RK_IMAGE_RC(save_hover)),
-                                 RkButton::State::Pressed);
-        RK_ACT_BINDL(saveFileButton, pressed, RK_ACT_ARGS(), [=,this](){saveFileButton->setPressed(false);
-                                                             action saveFile();});
-        mainLayout->addWidget(saveFileButton);
-        addSeparator(mainLayout, 5);
-        exportFileButton = new GeonkickButton(this);
-        exportFileButton->setSize(29, 10);
-        saveFileButton->setType(RkButton::ButtonType::ButtonCheckable);
-        exportFileButton->setImage(RkImage(exportFileButton->size(), RK_IMAGE_RC(export)),
-                                 RkButton::State::Unpressed);
-        exportFileButton->setImage(RkImage(exportFileButton->size(), RK_IMAGE_RC(export_hover)),
-                                 RkButton::State::UnpressedHover);
-        exportFileButton->setImage(RkImage(exportFileButton->size(), RK_IMAGE_RC(export_hover)),
-                                RkButton::State::Pressed);
-        RK_ACT_BINDL(exportFileButton, pressed, RK_ACT_ARGS(), [=,this](){exportFileButton->setPressed(false);
-                                                               action openExport();});
-        mainLayout->addWidget(exportFileButton);
-        addSeparator(mainLayout, 5);
+        settingsButton->show();
+        RK_ACT_BIND(settingsButton, pressed, RK_ACT_ARGS(), this, showSettings());
+        mainLayout->addWidget(settingsButton);
 
+        addSeparator(mainLayout, 5);
 	auto playButton = new RkButton(this);
         playButton->setType(RkButton::ButtonType::ButtonPush);
         playButton->setSize(21, 18);
@@ -178,15 +145,15 @@ TopBar::TopBar(GeonkickWidget *parent, GeonkickModel *model)
                              RkButton::State::UnpressedHover);
         playButton->setImage(RkImage(playButton->size(), RK_IMAGE_RC(play_pressed)),
                              RkButton::State::Pressed);
-        RK_ACT_BIND(playButton, pressed, RK_ACT_ARGS(), geonkickModel->api(), playKick());
+        RK_ACT_BIND(playButton, pressed, RK_ACT_ARGS(), geonkickModel->getDspProxy(), playKick());
 	playButton->show();
         mainLayout->addWidget(playButton);
         addSeparator(mainLayout, 5);
 
-#ifndef GEONKICK_LIMITED_VERSION
+#ifndef GEONKICK_BASIC_VERSION
         createLyersButtons(mainLayout);
         addSeparator(mainLayout, 5);
-#endif // GEONKICK_LIMITED_VERSION
+#endif // GEONKICK_BASIC_VERSION
 
         auto resetButton = new RkButton(this);
         resetButton->setSize(33, 18);
@@ -215,15 +182,19 @@ TopBar::TopBar(GeonkickWidget *parent, GeonkickModel *model)
         tuneCheckbox->setImage(RkImage(tuneCheckbox->size(), RK_IMAGE_RC(tune_checkbox_hover)),
                                RkButton::State::UnpressedHover);
         tuneCheckbox->show();
-        RK_ACT_BIND(tuneCheckbox, toggled, RK_ACT_ARGS(bool b), geonkickModel->api(),
-		    tuneAudioOutput(geonkickModel->api()->currentPercussion(), b));
+        RK_ACT_BIND(tuneCheckbox, toggled, RK_ACT_ARGS(bool b), geonkickModel->getDspProxy(),
+		    tuneAudioOutput(geonkickModel->getDspProxy()->currentPercussion(), b));
         mainLayout->addWidget(tuneCheckbox);
         addSeparator(mainLayout);
 
         presetNavigator = new PresetNavigator(this, geonkickModel->getPresetsModel());
         mainLayout->addWidget(presetNavigator);
 
+        // Instrument name
         addSeparator(mainLayout);
+        mainLayout->addWidget(createInstrumentNameLabel());
+        addSeparator(mainLayout);
+
         midiKeyButton = new GeonkickButton(this);
         midiKeyButton->setTextColor({200, 200, 200});
         midiKeyButton->setType(RkButton::ButtonType::ButtonUncheckable);
@@ -240,7 +211,6 @@ TopBar::TopBar(GeonkickWidget *parent, GeonkickModel *model)
                     showMidiPopup());
         mainLayout->addWidget(midiKeyButton);
 
-#ifdef GEONKICK_SINGLE
         // Midi channel
         addSeparator(mainLayout);
         midiChannelSpinBox = new RkSpinBox(this);
@@ -280,7 +250,6 @@ TopBar::TopBar(GeonkickWidget *parent, GeonkickModel *model)
                      [=, this](bool toggled) {
                              geonkickModel->getKitModel()->currentPercussion()->enableNoteOff(toggled);
                      } );
-#endif // GEONKICK_SINGLE
 
         // Controls button
         addSeparator(mainLayout);
@@ -320,69 +289,12 @@ TopBar::TopBar(GeonkickWidget *parent, GeonkickModel *model)
                     mainLayout->addWidget(kitButton);
 #endif // GEONKICK_SINGLE
 
-        // Presets button
-        addSeparator(mainLayout);
-        presetsButton = new GeonkickButton(this);
-        presetsButton->setPressed(viewState()->getMainView() == ViewState::View::Presets);
-        presetsButton->setFixedSize(45, 20);
-        presetsButton->setImage(RkImage(presetsButton->size(), RK_IMAGE_RC(topmenu_presets_off)),
-                               RkButton::State::Unpressed);
-        presetsButton->setImage(RkImage(presetsButton->size(), RK_IMAGE_RC(topmenu_presets_active)),
-                               RkButton::State::Pressed);
-        presetsButton->setImage(RkImage(presetsButton->size(), RK_IMAGE_RC(topmenu_presets_hover)),
-                               RkButton::State::UnpressedHover);
-        presetsButton->show();
-        RK_ACT_BIND(presetsButton, pressed, RK_ACT_ARGS(),
-                    viewState(), setMainView(ViewState::View::Presets));
-        RK_ACT_BIND(viewState(), mainViewChanged, RK_ACT_ARGS(ViewState::View view),
-                    presetsButton, setPressed(view == ViewState::View::Presets));
-        mainLayout->addWidget(presetsButton);
-
-        // Samples button
-        addSeparator(mainLayout);
-        samplesButton = new GeonkickButton(this);
-        samplesButton->setPressed(viewState()->getMainView() == ViewState::View::Samples);
-        samplesButton->setFixedSize(54, 20);
-        samplesButton->setImage(RkImage(samplesButton->size(), RK_IMAGE_RC(topmenu_samples_off)),
-                                RkButton::State::Unpressed);
-        samplesButton->setImage(RkImage(samplesButton->size(), RK_IMAGE_RC(topmenu_samples_active)),
-                                RkButton::State::Pressed);
-        samplesButton->setImage(RkImage(samplesButton->size(), RK_IMAGE_RC(topmenu_samples_hover)),
-                                RkButton::State::UnpressedHover);
-        samplesButton->show();
-        RK_ACT_BIND(samplesButton, pressed, RK_ACT_ARGS(),
-                    viewState(), setMainView(ViewState::View::Samples));
-        RK_ACT_BIND(viewState(), mainViewChanged, RK_ACT_ARGS(ViewState::View view),
-                    samplesButton, setPressed(view == ViewState::View::Samples));
-        mainLayout->addWidget(samplesButton);
-
-        // Setting button
-        addSeparator(mainLayout);
-        auto settingsButton = new GeonkickButton(this);
-        settingsButton->setPressed(viewState()->getMainView() == ViewState::View::Settings);
-        settingsButton->setFixedSize(48, 20);
-        settingsButton->setImage(RkImage(settingsButton->size(),
-                                         RK_IMAGE_RC(topmenu_settings_off)),
-                                 RkButton::State::Unpressed);
-        settingsButton->setImage(RkImage(settingsButton->size(),
-                                         RK_IMAGE_RC(topmenu_settings_active)),
-                                 RkButton::State::Pressed);
-        settingsButton->setImage(RkImage(settingsButton->size(),
-                                         RK_IMAGE_RC(topmenu_settings_hover)),
-                                 RkButton::State::UnpressedHover);
-        settingsButton->show();
-        RK_ACT_BIND(settingsButton, pressed, RK_ACT_ARGS(),
-                    viewState(), setMainView(ViewState::View::Settings));
-        RK_ACT_BIND(viewState(), mainViewChanged, RK_ACT_ARGS(ViewState::View view),
-                    settingsButton, setPressed(view == ViewState::View::Settings));
-        mainLayout->addWidget(settingsButton);
-
         RK_ACT_BIND(geonkickModel->getKitModel(),
                     modelUpdated,
                     RK_ACT_ARGS(),
                     this, updateGui());
         RK_ACT_BINDL(geonkickModel->getKitModel(),
-                     percussionUpdated,
+                     instrumentUpdated,
                      RK_ACT_ARGS(PercussionModel* model),
                      [=, this](PercussionModel* model) {
                              if (model->isSelected())
@@ -403,7 +315,7 @@ void TopBar::addSeparator(RkContainer *mainLayout, int width)
         mainLayout->addSpace(width);
 }
 
-#ifndef GEONKICK_LIMITED_VERSION
+#ifndef GEONKICK_BASIC_VERSION
 void TopBar::createLyersButtons(RkContainer *mainLayout)
 {
         layer1Button = new GeonkickButton(this);
@@ -454,13 +366,36 @@ void TopBar::createLyersButtons(RkContainer *mainLayout)
         mainLayout->addWidget(layer3Button);
 
         RK_ACT_BIND(layer1Button, toggled, RK_ACT_ARGS(bool b),
-                    geonkickModel->api(), enbaleLayer(GeonkickApi::Layer::Layer1, b));
+                    geonkickModel->getDspProxy(), enbaleLayer(DspProxy::Layer::Layer1, b));
         RK_ACT_BIND(layer3Button, toggled, RK_ACT_ARGS(bool b),
-                    geonkickModel->api(), enbaleLayer(GeonkickApi::Layer::Layer3, b));
+                    geonkickModel->getDspProxy(), enbaleLayer(DspProxy::Layer::Layer3, b));
         RK_ACT_BIND(layer2Button, toggled, RK_ACT_ARGS(bool b),
-        geonkickModel->api(), enbaleLayer(GeonkickApi::Layer::Layer2, b));
+        geonkickModel->getDspProxy(), enbaleLayer(DspProxy::Layer::Layer2, b));
+}
+#endif // GEONKICK_BASIC_VERSION
+
+RkWidget* TopBar::createInstrumentNameLabel()
+{
+        instrumentName = new RkLineEdit(this);
+        instrumentName->setBackgroundColor({44, 44, 44});
+        instrumentName->setTextColor({180, 180, 180});
+        instrumentName->setCursorColor({180, 180, 180});
+        instrumentName->setSize(100, 20);
+        instrumentName->show();
+        RK_ACT_BINDL(instrumentName, editingFinished, RK_ACT_ARGS(),
+                     [=, this]() {
+                             auto currentInstrument = geonkickModel->getKitModel()->currentPercussion();
+                             if (!currentInstrument->setName(instrumentName->text()))
+                                     instrumentName->setText(currentInstrument->name());
+                     });
+        RK_ACT_BINDL(instrumentName, escapePressed, RK_ACT_ARGS(),
+                     [=, this]() {
+                             auto currentInstrument = geonkickModel->getKitModel()->currentPercussion();
+                             instrumentName->setText(currentInstrument->name());
+                     });
+
+        return instrumentName;
 }
-#endif // GEONKICK_LIMITED_VERSION
 
 void TopBar::setPresetName(const std::string &name)
 {
@@ -468,34 +403,33 @@ void TopBar::setPresetName(const std::string &name)
                 std::string preset = name;
                 preset.resize(15);
                 preset += "...";
-                //                presetNameLabel->setText(preset);
+                instrumentName->setText(preset);
         } else {
-                //                presetNameLabel->setText(name);
+                instrumentName->setText(name);
         }
+        instrumentName->moveCursorToEnd();
 }
 
 void TopBar::updateGui()
 {
-        auto api = geonkickModel->api();
-#ifndef GEONKICK_LIMITED_VERSION
-        layer1Button->setPressed(api->isLayerEnabled(GeonkickApi::Layer::Layer1));
-        layer2Button->setPressed(api->isLayerEnabled(GeonkickApi::Layer::Layer2));
-        layer3Button->setPressed(api->isLayerEnabled(GeonkickApi::Layer::Layer3));
-#endif // GEONKICK_LIMITED_VERSION
-        tuneCheckbox->setPressed(api->isAudioOutputTuned(api->currentPercussion()));
+        auto dsp = geonkickModel->getDspProxy();
+#ifndef GEONKICK_BASIC_VERSION
+        layer1Button->setPressed(dsp->isLayerEnabled(DspProxy::Layer::Layer1));
+        layer2Button->setPressed(dsp->isLayerEnabled(DspProxy::Layer::Layer2));
+        layer3Button->setPressed(dsp->isLayerEnabled(DspProxy::Layer::Layer3));
+#endif // GEONKICK_BASIC_VERSION
+        tuneCheckbox->setPressed(dsp->isAudioOutputTuned(dsp->currentPercussion()));
         setPresetName(geonkickModel->getKitModel()->currentPercussion()->name());
         auto kitModel = geonkickModel->getKitModel();
         midiKeyButton->setText(MidiKeyWidget::midiKeyToNote(kitModel->currentPercussion()->key()));
-#ifdef GEONKICK_SINGLE
-        auto percussionModel = geonkickModel->getKitModel()->currentPercussion();
-        auto nMidiChannels = percussionModel->numberOfMidiChannels();
+        auto instrumentModel = geonkickModel->getKitModel()->currentPercussion();
+        auto nMidiChannels = instrumentModel->numberOfMidiChannels();
         midiChannelSpinBox->clear();
         midiChannelSpinBox->addItem("Any");
         for (size_t i = 0; i < nMidiChannels; i++)
                 midiChannelSpinBox->addItem(std::to_string(i + 1));
-        midiChannelSpinBox->setCurrentIndex(percussionModel->midiChannel() + 1);
-        noteOffButton->setPressed(percussionModel->isNoteOffEnabled());
-#endif // GEONKICK_SINGLE
+        midiChannelSpinBox->setCurrentIndex(instrumentModel->midiChannel() + 1);
+        noteOffButton->setPressed(instrumentModel->isNoteOffEnabled());
 }
 
 void TopBar::showMidiPopup()
@@ -503,7 +437,7 @@ void TopBar::showMidiPopup()
         auto kitModel = geonkickModel->getKitModel();
         auto midiPopup = new MidiKeyWidget(dynamic_cast<GeonkickWidget*>(getTopWidget()),
                                            kitModel->currentPercussion());
-        midiPopup->setPosition(midiKeyButton->x() - 170, y() + 35);
+        midiPopup->setPosition(150, y() + 35);
         RK_ACT_BIND(midiPopup,
                     isAboutToClose,
                     RK_ACT_ARGS(),
@@ -511,3 +445,14 @@ void TopBar::showMidiPopup()
                     setPressed(false));
         midiPopup->show();
 }
+
+void TopBar::showSettings()
+{
+        settingsButton->setPressed(false);
+        auto settingsPopup = new SettingsWidget(dynamic_cast<GeonkickWidget*>(getTopWidget()),
+                                                geonkickModel->getDspProxy());
+        settingsPopup->setPosition((getTopWidget()->width()
+                                    - settingsPopup->width()) / 2 - 120,
+                                   50);
+        settingsPopup->show();
+}
