@@ -1,6 +1,6 @@
 /**
  * File name: ViewState.h
- * Project: Geonkick (A kick synthesizer)
+ * Project: Geonkick (A percussive synthesizer)
  *
  * Copyright (C) 2020 Iurie Nistor 
  *
@@ -25,10 +25,10 @@
 #define GEONGKICK_VIEW_STATE_H
 
 #include "globals.h"
-#include "geonkick_api.h"
+#include "DspProxy.h"
 #include "envelope.h"
 
-#include <RkObject.h>
+#include "RkObject.h"
 
 class ViewState: public RkObject {
  public:
@@ -37,10 +37,9 @@ class ViewState: public RkObject {
                 Kit      = 1,
                 Presets  = 2,
                 Samples  = 3,
-                Settings = 4
         };
 
-        using Oscillator = GeonkickApi::OscillatorType;
+        using Oscillator = DspProxy::OscillatorType;
 
         ViewState(RkObject *parent);
         RK_DECL_ACT(mainViewChanged,
