@@ -1,6 +1,6 @@
 /**
  * File name: controls_widget.h
- * Project: Geonkick (A kick synthesizer)
+ * Project: Geonkick (A percussive synthesizer)
  *
  * Copyright (C) 2020 Iurie Nistor
  *
@@ -26,24 +26,19 @@
 
 #include "geonkick_widget.h"
 
-class GeonkickApi;
+class DspProxy;
 class Oscillator;
 class EnvelopeWidget;
-class EffectsGroupBox;
 class GeonkickModel;
 
 class ControlsWidget: public GeonkickWidget
 {
  public:
-        ControlsWidget(GeonkickWidget *parent,
-                       GeonkickModel* model,
-                       const std::vector<std::unique_ptr<Oscillator>> &oscillators);
-        ~ControlsWidget() = default;
+        ControlsWidget(GeonkickWidget *parent, GeonkickModel* model);
         RK_DECL_ACT(updateGui, updateGui(), RK_ARG_TYPE(), RK_ARG_VAL());
 
  private:
         GeonkickModel *geonkickModel;
-        EffectsGroupBox *effectsWidget;
 };
 
 #endif // CONTROLS_WIDGET_H
