@@ -2,7 +2,7 @@
  * File name: envelope.c
  * Project: Geonkick (A kick synthesizer)
  *
- * Copyright (C) 2017 Iurie Nistor 
+ * Copyright (C) 2017 Iurie Nistor
  *
  * This file is part of Geonkick.
  *
@@ -29,6 +29,7 @@ gkick_envelope_create(void)
 	struct gkick_envelope *envelope;
 	envelope = (struct gkick_envelope*)calloc(1, sizeof(struct gkick_envelope));
 	envelope->apply_type = GEONKICK_ENVELOPE_APPLY_LINEAR;
+        envelope->curve_type = GEONKICK_ENVELOPE_CURVE_LINEAR;
 	return envelope;
 }
 
@@ -44,87 +45,132 @@ gkick_envelope_get_apply_type(const struct gkick_envelope* envelope)
         return envelope->apply_type;
 }
 
-gkick_real
-gkick_envelope_get_value(const struct gkick_envelope* envelope, gkick_real xm)
+void gkick_envelope_set_curve_type(struct gkick_envelope* envelope,
+				   enum gkick_envelope_curve_type type)
 {
-	gkick_real x1;
-        gkick_real y1 = 0.0f;
-        gkick_real x2;
-        gkick_real y2 = 0.0f;
-        gkick_real ym;
+        envelope->curve_type = type;
+}
 
-	struct gkick_envelope_point *p;
-	if (envelope == NULL ||
-	    (envelope->first == NULL
-	     || envelope->last == NULL)) {
-		return 0.0f;
-	}
+enum gkick_envelope_curve_type
+gkick_envelope_get_curve_type(struct gkick_envelope* envelope)
+{
+        return envelope->curve_type;
+}
 
-	if (xm < envelope->first->x || xm > envelope->last->x) {
-		return 0.0f;
-	} else if (fabsl(xm - envelope->first->x) < DBL_EPSILON) {
-                return envelope->first->y;
-        } else if (fabsl(envelope->last->x - xm) < DBL_EPSILON) {
-                return envelope->last->y;
+static gkick_real linear_interpolate(gkick_real x0,
+                                     gkick_real y0,
+                                     gkick_real x1,
+                                     gkick_real y1,
+                                     gkick_real x)
+{
+    return y0 + (y1 - y0) * ((x - x0) / (x1 - x0));
+}
+
+static gkick_real bezier_x(gkick_real t,
+                           gkick_real x0,
+                           gkick_real x1,
+                           gkick_real x2,
+                           gkick_real x3)
+{
+        const gkick_real one_minus_t = 1.0f - t;
+        return pow(one_minus_t, 3.0f) * x0
+                + 3.0f * pow(one_minus_t, 2.0f) * t * x1
+                + 3.0f * one_minus_t * pow(t, 2.0f) * x2
+                + pow(t, 3.0f) * x3;
+}
+
+static gkick_real bezier_y(gkick_real t,
+                           gkick_real y0,
+                           gkick_real y1,
+                           gkick_real y2,
+                           gkick_real y3)
+{
+        const gkick_real one_minus_t = 1.0f - t;
+        return pow(one_minus_t, 3.0f) * y0
+                + 3.0f * pow(one_minus_t, 2.0f) * t * y1
+                + 3.0f * one_minus_t * pow(t, 2.0f) * y2
+                + pow(t, 3.0f) * y3;
+}
+
+static gkick_real find_t(const struct gkick_envelope_point *p1,
+                         const struct gkick_envelope_point *p2,
+                         const struct gkick_envelope_point *p3,
+                         gkick_real xm)
+{
+        gkick_real t0 = 0.0f;
+        gkick_real t1 = 1.0f;
+        gkick_real t;
+        const gkick_real p1_x = p1->x;
+        const gkick_real p2_x = p2->x;
+        const gkick_real p3_x = p3->x;
+        // Binary search to find t corresponding to xm.
+        while (t1 - t0 > 1e-6) {
+                t = (t0 + t1) * 0.5f;
+                gkick_real x_t = bezier_x(t, p1_x, p1_x, p2_x, p3_x);
+                if (x_t < xm)
+                        t0 = t;
+                else
+                        t1 = t;
         }
-
-        x2 = x1 = xm;
-	p = envelope->first;
-	while (p) {
-                if (fabs(xm - p->x) < DBL_EPSILON) {
-                        return p->y;
-                } else if (xm < p->x) {
-			x2 = p->x;
-			y2 = p->y;
-			break;
-		}
-		p = p->next;
-	}
-
-	p = envelope->last;
-	while (p) {
-                if (fabs(xm - p->x) < DBL_EPSILON) {
-                        return p->y;
-                } else if (p->x < xm) {
-			x1 = p->x;
-			y1 = p->y;
-			break;
-		}
-		p = p->prev;
-	}
-
-	if (fabsl(x2 - x1) < DBL_EPSILON)
-	        ym = y1;
-	else
-	        ym = (y1 * (x2 - xm) + y2 * (xm - x1)) / (x2 - x1);
-
-	return ym;
+        return t;
 }
 
-struct gkick_envelope_point*
-gkick_envelope_add_point(struct gkick_envelope *envelope,
-                         float x,
-                         float y)
+static gkick_real bezier(struct gkick_envelope_point *p1,
+                         struct gkick_envelope_point *p2,
+                         struct gkick_envelope_point *p3,
+                         float xm)
 {
-	struct gkick_envelope_point *point;
-	if (envelope == NULL)
-		return NULL;
-	point = (struct gkick_envelope_point*)malloc(sizeof(struct gkick_envelope_point));
-	if (point == NULL)
-		return NULL;
-	point->x = x;
-	point->y = y;
-        point->next = point->prev = NULL;
+    if (fabsl(p3->x - p1->x) < DBL_EPSILON)
+            return p1->y;
 
-	if (envelope->first == NULL ||  envelope->last == NULL)
-		envelope->first = envelope->last = point;
-	else
-		gkick_envelope_add_sorted(envelope, point);
-	envelope->npoints++;
-	return point;
+    gkick_real t = find_t(p1, p2, p3, xm);
+    return bezier_y(t, p1->y, p1->y, p2->y, p3->y);
 }
 
-void gkick_envelope_add_sorted(struct gkick_envelope *envelope,
+gkick_real
+gkick_envelope_get_value(const struct gkick_envelope* envelope,
+                         gkick_real xm)
+{
+    if (envelope == NULL
+        || envelope->npoints < 2
+        || envelope->first == NULL
+        || envelope->last == NULL)
+        return 0.0f;
+
+    struct gkick_envelope_point *p = envelope->first;
+    if (xm < p->x || xm > envelope->last->x)
+            return 0.0f;
+    else if (fabsl(xm - p->x) < DBL_EPSILON)
+            return p->y;
+    else if (fabsl(envelope->last->x - xm) < DBL_EPSILON)
+            return envelope->last->y;
+
+    if (envelope->npoints == 2)
+        return linear_interpolate(envelope->first->x, envelope->first->y,
+                                  envelope->last->x, envelope->last->y, xm);
+
+    struct gkick_envelope_point *p1 = NULL, *p2 = NULL;
+    while (p) {
+            p1 = p;
+            p2 = p->next;
+            if (p2 && p1->x <= xm && xm <= p2->x)
+                    break;
+            p = p->next;
+    }
+
+    if (!p1 || !p2)
+            return 0.0f;
+
+    if (p1->is_control && p1->prev)
+            return bezier(p1->prev, p1, p2, xm);
+
+    if (p2->is_control && p2->next)
+            return bezier(p1, p2, p2->next, xm);
+
+    return linear_interpolate(p1->x, p1->y, p2->x, p2->y, xm);
+}
+
+static void gkick_envelope_add_sorted(struct gkick_envelope *envelope,
 			       struct gkick_envelope_point *point)
 {
 	struct gkick_envelope_point *p;
@@ -153,6 +199,29 @@ void gkick_envelope_add_sorted(struct gkick_envelope *envelope,
 	}
 }
 
+struct gkick_envelope_point*
+gkick_envelope_add_point(struct gkick_envelope *envelope,
+                         const struct gkick_envelope_point_info *point_info)
+{
+	struct gkick_envelope_point *point;
+	if (envelope == NULL)
+		return NULL;
+	point = (struct gkick_envelope_point*)malloc(sizeof(struct gkick_envelope_point));
+	if (point == NULL)
+		return NULL;
+	point->x = point_info->x;
+	point->y = point_info->y;
+        point->is_control = point_info->control_point;
+        point->next = point->prev = NULL;
+
+	if (envelope->first == NULL ||  envelope->last == NULL)
+		envelope->first = envelope->last = point;
+	else
+		gkick_envelope_add_sorted(envelope, point);
+	envelope->npoints++;
+	return point;
+}
+
 void gkick_envelope_destroy(struct gkick_envelope *envelope)
 {
 	struct gkick_envelope_point *point;
@@ -171,11 +240,11 @@ void gkick_envelope_destroy(struct gkick_envelope *envelope)
 
 void
 gkick_envelope_get_points(struct gkick_envelope *env,
-			  gkick_real **buff,
+			  struct gkick_envelope_point_info **buff,
 			  size_t *npoints)
 {
         struct gkick_envelope_point *p;
-        gkick_real *points;
+        struct gkick_envelope_point_info *points_info;
         size_t i;
 
         if (buff == NULL)
@@ -185,23 +254,24 @@ gkick_envelope_get_points(struct gkick_envelope *env,
         if (env->npoints < 1)
                 return;
 
-        points = (gkick_real *)calloc(1, sizeof(gkick_real) * (2 * env->npoints));
+        points_info = (struct gkick_envelope_point_info*)calloc(1, sizeof(struct gkick_envelope_point_info) * env->npoints);
         p = env->first;
         i = 0;
         while (p) {
-                points[i]     = p->x;
-                points[i + 1] = p->y;
+                points_info[i].x = p->x;
+                points_info[i].y = p->y;
+                points_info[i].control_point = p->is_control;
                 p = p->next;
-                i += 2;
+                i++;
         }
 
-        *buff = points;
+        *buff = points_info;
         *npoints = env->npoints;
 }
 
 void
 gkick_envelope_set_points(struct gkick_envelope *env,
-			  const gkick_real *buff,
+			  const struct gkick_envelope_point_info *buff,
 			  size_t npoints)
 {
         if (env == NULL || buff == NULL)
@@ -209,7 +279,7 @@ gkick_envelope_set_points(struct gkick_envelope *env,
 
         gkick_envelope_clear(env);
         for (size_t i = 0; i < npoints; i++)
-                gkick_envelope_add_point(env, buff[2 * i], buff[2 * i + 1]);
+                gkick_envelope_add_point(env, &buff[i]);
 }
 
 void gkick_envelope_clear(struct gkick_envelope* env)
@@ -255,8 +325,7 @@ gkick_envelope_remove_point(struct gkick_envelope *env, size_t index)
 void
 gkick_envelope_update_point(struct gkick_envelope *env,
 			    size_t index,
-			    gkick_real x,
-			    gkick_real y)
+                            const struct gkick_envelope_point_info *point_info)
 {
         if (env == NULL || index >= env->npoints)
                 return;
@@ -265,8 +334,9 @@ gkick_envelope_update_point(struct gkick_envelope *env,
         size_t i = 0;
         while (p) {
                 if (i == index) {
-                        p->x = x;
-                        p->y = y;
+                        p->x = point_info->x;
+                        p->y = point_info->y;
+                        p->is_control = point_info->control_point;
                         break;
                 }
                 p = p->next;
