@@ -2,7 +2,7 @@
  * File name: geonkick.c
  * Project: Geonkick (A kick synthesizer)
  *
- * Copyright (C) 2017 Iurie Nistor 
+ * Copyright (C) 2017 Iurie Nistor
  *
  * This file is part of Geonkick.
  *
@@ -27,6 +27,7 @@
 #include "audio_output.h"
 #include "envelope.h"
 #include "mixer.h"
+#include "oscillator.h"
 
 #include <unistd.h>
 #include <stdlib.h>
@@ -40,6 +41,7 @@ geonkick_create(struct geonkick **kick, int sample_rate)
 	*kick = (struct geonkick*)calloc(1, sizeof(struct geonkick));
 	if (*kick == NULL)
 		return GEONKICK_ERROR_MEM_ALLOC;
+
 	strcpy((*kick)->name, "Geonkick");
 	(*kick)->sample_rate = sample_rate;
         (*kick)->synthesis_on = false;
@@ -70,7 +72,7 @@ geonkick_create(struct geonkick **kick, int sample_rate)
 
         for (size_t i = 0; i < GEONKICK_MAX_INSTRUMENTS; i++) {
                 gkick_synth_set_output((*kick)->synths[i], (*kick)->audio->audio_outputs[i]);
-                geonkick_set_percussion_channel(*kick, i, i % GEONKICK_MAX_CHANNELS);
+                geonkick_set_instrument_channel(*kick, i, i % GEONKICK_MAX_CHANNELS);
         }
 
         if (!geonkick_worker_created()) {
@@ -102,7 +104,6 @@ void geonkick_free(struct geonkick **kick)
                 geonkick_worker_remove_instance(*kick);
                 if (geonkick_worker_reference_count() == 0)
                         geonkick_worker_destroy();
-                gkick_log_debug("ref count: %d", geonkick_worker_reference_count());
                 for (size_t i = 0; i < GEONKICK_MAX_INSTRUMENTS; i++)
                         gkick_synth_free(&((*kick)->synths[i]));
                 gkick_audio_free(&((*kick)->audio));
@@ -165,10 +166,9 @@ enum geonkick_error
 geonkick_osc_envelope_add_point(struct geonkick *kick,
 				size_t osc_index,
 				size_t env_index,
-				gkick_real x,
-				gkick_real y)
+                                struct gkick_envelope_point_info *point_info)
 {
-        if (kick == NULL) {
+        if (kick == NULL || point_info == NULL) {
                 gkick_log_error("wrong arguments");
                 return GEONKICK_ERROR;
         }
@@ -176,7 +176,7 @@ geonkick_osc_envelope_add_point(struct geonkick *kick,
         enum geonkick_error res;
         res = gkick_synth_osc_env_add_point(kick->synths[kick->per_index],
                                             osc_index,
-                                            env_index, x, y);
+                                            env_index, point_info);
         if (res == GEONKICK_OK && kick->synths[kick->per_index]->buffer_update)
                 geonkick_wakeup(kick);
         return res;
@@ -192,7 +192,7 @@ enum geonkick_error
 geonkick_osc_envelope_get_points(struct geonkick *kick,
 				 size_t osc_index,
 				 size_t env_index,
-				 gkick_real **buf,
+				 struct gkick_envelope_point_info **buf,
 				 size_t *npoints)
 {
         if (kick == NULL || buf == NULL || npoints == NULL) {
@@ -211,7 +211,7 @@ enum geonkick_error
 geonkick_osc_envelope_set_points(struct geonkick *kick,
                                  size_t osc_index,
                                  size_t env_index,
-                                 const gkick_real *buff,
+                                 const struct gkick_envelope_point_info *buff,
                                  size_t npoints)
 {
         if (kick == NULL || buff == NULL || npoints == 0) {
@@ -252,8 +252,7 @@ geonkick_osc_envelope_update_point(struct geonkick *kick,
 				   size_t osc_index,
 				   size_t env_index,
   				   size_t index,
-				   gkick_real x,
-				   gkick_real y)
+                                   struct gkick_envelope_point_info *point_info)
 {
         if (kick == NULL) {
                 gkick_log_error("wrong arguments");
@@ -264,7 +263,8 @@ geonkick_osc_envelope_update_point(struct geonkick *kick,
         res = gkick_synth_osc_env_update_point(kick->synths[kick->per_index],
                                                osc_index,
                                                env_index,
-                                               index, x, y);
+                                               index,
+                                               point_info);
         if (res == GEONKICK_OK && kick->synths[kick->per_index]->buffer_update)
                 geonkick_wakeup(kick);
 
@@ -272,7 +272,7 @@ geonkick_osc_envelope_update_point(struct geonkick *kick,
 }
 
 enum geonkick_error
-geonkick_osc_envelope_set_apply_type(struct geonkick *kick, 
+geonkick_osc_envelope_set_apply_type(struct geonkick *kick,
 				     size_t osc_index,
 				     size_t env_index,
 				     enum gkick_envelope_apply_type apply_type)
@@ -329,6 +329,30 @@ geonkick_osc_is_fm(struct geonkick *kick, size_t index, bool *is_fm)
         return gkick_synth_osc_is_fm(kick->synths[kick->per_index], index, is_fm);
 }
 
+enum geonkick_error
+geonkick_osc_set_fm_k(struct geonkick *kick, size_t index, gkick_real k)
+{
+        if (kick == NULL) {
+                gkick_log_error("wrong arguments");
+                return GEONKICK_ERROR;
+        }
+
+        struct gkick_synth *synth = kick->synths[kick->per_index];
+        gkick_synth_lock(synth);
+        struct gkick_oscillator* osc;
+        osc = gkick_synth_get_oscillator(synth, index);
+	if (osc == NULL) {
+		gkick_log_error("can't get oscillator");
+		gkick_synth_unlock(synth);
+		return GEONKICK_ERROR;
+	}
+
+        osc->fm_k = k;
+	gkick_synth_unlock(synth);
+
+        return GEONKICK_OK;
+}
+
 enum geonkick_error
 geonkick_set_osc_function(struct geonkick *kick,
 			  size_t osc_index,
@@ -593,7 +617,7 @@ geonkick_get_kick_filter_type(struct geonkick *kick,
 enum geonkick_error
 geonkick_kick_envelope_get_points(struct geonkick *kick,
                                   enum geonkick_envelope_type env_type,
-                                  gkick_real **buf,
+                                  struct gkick_envelope_point_info **buf,
                                   size_t *npoints)
 {
         if (kick == NULL || buf == NULL || npoints == NULL) {
@@ -609,7 +633,7 @@ geonkick_kick_envelope_get_points(struct geonkick *kick,
 enum geonkick_error
 geonkick_kick_envelope_set_points(struct geonkick *kick,
                                   enum geonkick_envelope_type env_type,
-                                  const gkick_real *buff,
+                                  const struct gkick_envelope_point_info *buff,
                                   size_t npoints)
 {
         if (kick == NULL || buff == NULL || npoints == 0)
@@ -628,8 +652,7 @@ geonkick_kick_envelope_set_points(struct geonkick *kick,
 enum geonkick_error
 geonkick_kick_add_env_point(struct geonkick *kick,
                             enum geonkick_envelope_type env_type,
-                            gkick_real x,
-                            gkick_real y)
+                            const struct gkick_envelope_point_info *point_info)
 {
         if (kick == NULL) {
                 gkick_log_error("wrong arguments");
@@ -638,7 +661,7 @@ geonkick_kick_add_env_point(struct geonkick *kick,
         enum geonkick_error res;
         res = gkick_synth_kick_add_env_point(kick->synths[kick->per_index],
                                              env_type,
-                                             x, y);
+                                             point_info);
         if (res == GEONKICK_OK && kick->synths[kick->per_index]->buffer_update)
                 geonkick_wakeup(kick);
         return res;
@@ -666,8 +689,7 @@ enum geonkick_error
 geonkick_kick_update_env_point(struct geonkick *kick,
                                enum geonkick_envelope_type env_type,
                                size_t index,
-                               gkick_real x,
-                               gkick_real y)
+                               const struct gkick_envelope_point_info *point_info)
 {
         if (kick == NULL) {
                 gkick_log_error("wrong arguments");
@@ -676,14 +698,14 @@ geonkick_kick_update_env_point(struct geonkick *kick,
         enum geonkick_error res;
         res = gkick_synth_kick_update_env_point(kick->synths[kick->per_index],
                                                 env_type,
-                                                index, x, y);
+                                                index, point_info);
         if (res == GEONKICK_OK && kick->synths[kick->per_index]->buffer_update)
                 geonkick_wakeup(kick);
         return res;
 }
 
 enum geonkick_error
-geonkick_kick_env_set_apply_type(struct geonkick *kick, 
+geonkick_kick_env_set_apply_type(struct geonkick *kick,
 				 enum geonkick_envelope_type env_type,
 				 enum gkick_envelope_apply_type apply_type)
 {
@@ -1136,7 +1158,7 @@ geonkick_audio_process(struct geonkick *kick,
 }
 
 enum geonkick_error
-geonkick_distortion_enable(struct geonkick *kick, int enable)
+geonkick_distortion_enable(struct geonkick *kick, bool enable)
 {
         if (kick == NULL) {
                 gkick_log_error("wrong arguments");
@@ -1152,7 +1174,7 @@ geonkick_distortion_enable(struct geonkick *kick, int enable)
 
 enum geonkick_error
 geonkick_distortion_is_enabled(struct geonkick *kick,
-                               int *enabled)
+                               bool *enabled)
 {
         if (kick == NULL || enabled == NULL) {
                 gkick_log_error("wrong arguments");
@@ -1162,6 +1184,34 @@ geonkick_distortion_is_enabled(struct geonkick *kick,
                                                  enabled);
 }
 
+enum geonkick_error
+geonkick_distortion_set_type(struct geonkick *kick,
+                             enum gkick_distortion_type type)
+{
+        if (kick == NULL) {
+                gkick_log_error("wrong arguments");
+                return GEONKICK_ERROR;
+        }
+        enum geonkick_error res;
+        res = gkick_synth_distortion_set_type(kick->synths[kick->per_index],
+                                              type);
+        if (res == GEONKICK_OK && kick->synths[kick->per_index]->buffer_update)
+                geonkick_wakeup(kick);
+        return res;
+}
+
+enum geonkick_error
+geonkick_distortion_get_type(struct geonkick *kick,
+                             enum gkick_distortion_type *type)
+{
+        if (kick == NULL || type == NULL) {
+                gkick_log_error("wrong arguments");
+                return GEONKICK_ERROR;
+        }
+        return gkick_synth_distortion_get_type(kick->synths[kick->per_index],
+                                               type);
+}
+
 enum geonkick_error
 geonkick_distortion_set_in_limiter(struct geonkick *kick,
                                    gkick_real limit)
@@ -1192,30 +1242,30 @@ geonkick_distortion_get_in_limiter(struct geonkick *kick,
 }
 
 enum geonkick_error
-geonkick_distortion_set_volume(struct geonkick *kick,
-                               gkick_real volume)
+geonkick_distortion_set_out_limiter(struct geonkick *kick,
+                                    gkick_real value)
 {
         if (kick == NULL) {
                 gkick_log_error("wrong arguments");
                 return GEONKICK_ERROR;
         }
         enum geonkick_error res;
-        res = gkick_synth_distortion_set_volume(kick->synths[kick->per_index],
-                                                volume);
+        res = gkick_synth_distortion_set_out_limiter(kick->synths[kick->per_index],
+                                                     value);
         if (res == GEONKICK_OK && kick->synths[kick->per_index]->buffer_update)
                 geonkick_wakeup(kick);
         return res;
 }
 
 enum geonkick_error
-geonkick_distortion_get_volume(struct geonkick *kick, gkick_real *volume)
+geonkick_distortion_get_out_limiter(struct geonkick *kick, gkick_real *value)
 {
-        if (kick == NULL || volume == NULL) {
+        if (kick == NULL || value == NULL) {
                 gkick_log_error("wrong arguments");
                 return GEONKICK_ERROR;
         }
-        return gkick_synth_distortion_get_volume(kick->synths[kick->per_index],
-                                                 volume);
+        return gkick_synth_distortion_get_out_limiter(kick->synths[kick->per_index],
+                                                      value);
 }
 
 enum geonkick_error
@@ -1246,6 +1296,176 @@ geonkick_distortion_get_drive(struct geonkick *kick,
                                                 drive);
 }
 
+enum geonkick_error
+geonkick_osc_distortion_enable(struct geonkick *kick,
+                               size_t index,
+                               bool enable)
+{
+        struct gkick_oscillator* osc;
+        osc = gkick_synth_get_oscillator(kick->synths[kick->per_index], index);
+        if (!osc)
+                return GEONKICK_ERROR;
+        enum geonkick_error res;
+        res = gkick_distortion_enable(osc->distortion, enable);
+        if (res == GEONKICK_OK) {
+                kick->synths[kick->per_index]->buffer_update = true;
+                geonkick_wakeup(kick);
+        }
+        return res;
+}
+
+enum geonkick_error
+geonkick_osc_distortion_is_enabled(struct geonkick *kick,
+                                   size_t index,
+                                   bool *enabled)
+{
+        struct gkick_oscillator* osc;
+        osc = gkick_synth_get_oscillator(kick->synths[kick->per_index], index);
+        if (!osc)
+                return GEONKICK_ERROR;
+        return gkick_distortion_is_enabled(osc->distortion, enabled);
+}
+
+enum geonkick_error
+geonkick_osc_distortion_get_type(struct geonkick *kick,
+                                 size_t index,
+                                 enum gkick_distortion_type *type)
+{
+        struct gkick_oscillator* osc;
+        osc = gkick_synth_get_oscillator(kick->synths[kick->per_index], index);
+        if (!osc)
+                return GEONKICK_ERROR;
+        return gkick_distortion_get_type(osc->distortion, type);
+}
+
+enum geonkick_error
+geonkick_osc_distortion_set_type(struct geonkick *kick,
+                                 size_t index,
+                                 enum gkick_distortion_type type)
+{
+        struct gkick_oscillator* osc;
+        struct gkick_synth *synth = kick->synths[kick->per_index];
+        osc = gkick_synth_get_oscillator(synth, index);
+        if (!osc)
+                return GEONKICK_ERROR;
+        enum geonkick_error res;
+        res = gkick_distortion_set_type(osc->distortion, type);
+        if (res != GEONKICK_OK)
+                return res;
+        bool enabled = false;
+        gkick_distortion_is_enabled(osc->distortion, &enabled);
+        if (enabled) {
+                synth->buffer_update = true;
+                geonkick_wakeup(kick);
+        }
+        return res;
+}
+
+enum geonkick_error
+geonkick_osc_distortion_set_in_limiter(struct geonkick *kick,
+                                       size_t index,
+                                       gkick_real value)
+{
+        struct gkick_oscillator* osc;
+        struct gkick_synth *synth = kick->synths[kick->per_index];
+        osc = gkick_synth_get_oscillator(synth, index);
+        if (!osc)
+                return GEONKICK_ERROR;
+        enum geonkick_error res;
+        res = gkick_distortion_set_in_limiter(osc->distortion, value);
+        if (res != GEONKICK_OK)
+                return res;
+        bool enabled = false;
+        gkick_distortion_is_enabled(osc->distortion, &enabled);
+        if (enabled) {
+                synth->buffer_update = true;
+                geonkick_wakeup(kick);
+        }
+        return res;
+}
+
+enum geonkick_error
+geonkick_osc_distortion_get_in_limiter(struct geonkick *kick,
+                                       size_t index,
+                                       gkick_real *value)
+{
+        struct gkick_oscillator* osc;
+        osc = gkick_synth_get_oscillator(kick->synths[kick->per_index], index);
+        if (!osc)
+                return GEONKICK_ERROR;
+        return gkick_distortion_get_in_limiter(osc->distortion, value);
+}
+
+enum geonkick_error
+geonkick_osc_distortion_set_out_limiter(struct geonkick *kick,
+                                        size_t index,
+                                        gkick_real value)
+{
+        struct gkick_oscillator* osc;
+        struct gkick_synth *synth = kick->synths[kick->per_index];
+        osc = gkick_synth_get_oscillator(synth, index);
+        if (!osc)
+                return GEONKICK_ERROR;
+        enum geonkick_error res;
+        res = gkick_distortion_set_out_limiter(osc->distortion, value);
+        if (res != GEONKICK_OK)
+                return res;
+        bool enabled = false;
+        gkick_distortion_is_enabled(osc->distortion, &enabled);
+        if (enabled) {
+                synth->buffer_update = true;
+                geonkick_wakeup(kick);
+        }
+        return res;
+}
+
+enum geonkick_error
+geonkick_osc_distortion_get_out_limiter(struct geonkick *kick,
+                                        size_t index,
+                                        gkick_real *value)
+{
+        struct gkick_oscillator* osc;
+        osc = gkick_synth_get_oscillator(kick->synths[kick->per_index], index);
+        if (!osc)
+                return GEONKICK_ERROR;
+        return gkick_distortion_get_out_limiter(osc->distortion, value);
+}
+
+enum geonkick_error
+geonkick_osc_distortion_set_drive(struct geonkick *kick,
+                                  size_t index,
+                                  gkick_real drive)
+{
+        struct gkick_oscillator* osc;
+        struct gkick_synth *synth = kick->synths[kick->per_index];
+        osc = gkick_synth_get_oscillator(synth, index);
+        if (!osc)
+                return GEONKICK_ERROR;
+        enum geonkick_error res;
+        res = gkick_distortion_set_drive(osc->distortion, drive);
+        if (res != GEONKICK_OK)
+                return res;
+        bool enabled = false;
+        gkick_distortion_is_enabled(osc->distortion, &enabled);
+        if (enabled) {
+                synth->buffer_update = true;
+                geonkick_wakeup(kick);
+        }
+        return res;
+}
+
+enum geonkick_error
+geonkick_osc_distortion_get_drive(struct geonkick *kick,
+                                  size_t index,
+                                  gkick_real *drive)
+{
+        struct gkick_oscillator* osc;
+        osc = gkick_synth_get_oscillator(kick->synths[kick->per_index], index);
+        if (!osc)
+                return GEONKICK_ERROR;
+        return gkick_distortion_get_drive(osc->distortion, drive);
+}
+
 int geonkick_is_module_enabed(struct geonkick *kick,
                               enum GEONKICK_MODULE module)
 {
@@ -1406,7 +1626,7 @@ geonkick_get_osc_sample(struct geonkick *kick,
 }
 
 enum geonkick_error
-geonkick_set_current_percussion(struct geonkick *kick,
+geonkick_set_current_instrument(struct geonkick *kick,
                                 size_t index)
 {
         if (kick == NULL || index >= GEONKICK_MAX_INSTRUMENTS) {
@@ -1420,11 +1640,11 @@ geonkick_set_current_percussion(struct geonkick *kick,
 }
 
 enum geonkick_error
-geonkick_get_current_percussion(struct geonkick *kick,
+geonkick_get_current_instrument(const struct geonkick *kick,
                                 size_t *index)
 {
         if (kick == NULL || index == NULL) {
-                gkick_log_error("wrong arguments");
+                gkick_log_error("wrong arguments %d, %d", kick, index);
                 return GEONKICK_ERROR;
         }
 
@@ -1442,10 +1662,10 @@ void geonkick_process(struct geonkick *kick)
 }
 
 /**
- * Finds the first unused/free percussion index.
+ * Finds the first unused/free instrument index.
  */
 enum geonkick_error
-geonkick_unused_percussion(struct geonkick *kick,
+geonkick_unused_instrument(struct geonkick *kick,
                            int *index)
 {
         if (kick == NULL || index == NULL) {
@@ -1464,7 +1684,7 @@ geonkick_unused_percussion(struct geonkick *kick,
 }
 
 enum geonkick_error
-geonkick_enable_percussion(struct geonkick *kick,
+geonkick_enable_instrument(struct geonkick *kick,
                            size_t index,
                            bool enable)
 {
@@ -1478,7 +1698,7 @@ geonkick_enable_percussion(struct geonkick *kick,
 }
 
 enum geonkick_error
-geonkick_is_percussion_enabled(struct geonkick *kick,
+geonkick_is_instrument_enabled(struct geonkick *kick,
                                size_t index,
                                bool *enable)
 {
@@ -1570,13 +1790,13 @@ geonkick_ged_forced_midi_channel(struct geonkick *kick,
 }
 
 size_t
-geonkick_midi_channels_number(struct geonkick *kick)
+geonkick_midi_channels_number(void)
 {
         return GEONKICK_MAX_MIDI_CHANNELS;
 }
 
 enum geonkick_error
-geonkick_set_percussion_name(struct geonkick *kick,
+geonkick_set_instrument_name(struct geonkick *kick,
                              size_t id,
                              const char *name,
                              size_t size)
@@ -1598,7 +1818,7 @@ geonkick_set_percussion_name(struct geonkick *kick,
 }
 
 enum geonkick_error
-geonkick_get_percussion_name(struct geonkick *kick,
+geonkick_get_instrument_name(struct geonkick *kick,
                              size_t id,
                              char *name,
                              size_t size)
@@ -1632,7 +1852,7 @@ geonkick_instruments_number()
 }
 
 enum geonkick_error
-geonkick_set_percussion_channel(struct geonkick *kick,
+geonkick_set_instrument_channel(struct geonkick *kick,
                                 size_t id,
                                 size_t channel)
 {
@@ -1645,7 +1865,7 @@ geonkick_set_percussion_channel(struct geonkick *kick,
 }
 
 enum geonkick_error
-geonkick_get_percussion_channel(struct geonkick *kick,
+geonkick_get_instrument_channel(struct geonkick *kick,
                                 size_t id,
                                 size_t *channel)
 {
@@ -1658,7 +1878,7 @@ geonkick_get_percussion_channel(struct geonkick *kick,
 }
 
 enum geonkick_error
-geonkick_percussion_set_limiter(struct geonkick *kick,
+geonkick_instrument_set_limiter(struct geonkick *kick,
                                 size_t id,
                                 gkick_real val)
 {
@@ -1670,7 +1890,7 @@ geonkick_percussion_set_limiter(struct geonkick *kick,
 }
 
 enum geonkick_error
-geonkick_percussion_get_limiter(struct geonkick *kick,
+geonkick_instrument_get_limiter(struct geonkick *kick,
                                 size_t id,
                                 gkick_real *val)
 {
@@ -1682,7 +1902,7 @@ geonkick_percussion_get_limiter(struct geonkick *kick,
 }
 
 enum geonkick_error
-geonkick_percussion_mute(struct geonkick *kick,
+geonkick_instrument_mute(struct geonkick *kick,
                          size_t id,
                          bool b)
 {
@@ -1694,7 +1914,7 @@ geonkick_percussion_mute(struct geonkick *kick,
 }
 
 enum geonkick_error
-geonkick_percussion_is_muted(struct geonkick *kick,
+geonkick_instrument_is_muted(struct geonkick *kick,
                              size_t id,
                              bool *b)
 {
@@ -1706,7 +1926,7 @@ geonkick_percussion_is_muted(struct geonkick *kick,
 }
 
 enum geonkick_error
-geonkick_percussion_solo(struct geonkick *kick,
+geonkick_instrument_solo(struct geonkick *kick,
                          size_t id,
                          bool b)
 {
@@ -1718,7 +1938,7 @@ geonkick_percussion_solo(struct geonkick *kick,
 }
 
 enum geonkick_error
-geonkick_percussion_is_solo(struct geonkick *kick,
+geonkick_instrument_is_solo(struct geonkick *kick,
                             size_t id,
                             bool *b)
 {
@@ -1771,7 +1991,7 @@ geonkick_get_sample_preview_limiter(struct geonkick *kick, gkick_real *val)
 }
 
 enum geonkick_error
-geonkick_percussion_enable_note_off(struct geonkick *kick,
+geonkick_instrument_enable_note_off(struct geonkick *kick,
                                     size_t id,
                                     bool enable)
 {
@@ -1779,7 +1999,7 @@ geonkick_percussion_enable_note_off(struct geonkick *kick,
 }
 
 enum geonkick_error
-geonkick_percussion_note_off_enabled(struct geonkick *kick,
+geonkick_instrument_note_off_enabled(struct geonkick *kick,
                                      size_t id,
                                      bool *enabled)
 {
@@ -1821,3 +2041,93 @@ size_t geonkick_layers_number(void)
 {
         return GKICK_OSC_GROUPS_NUMBER;
 }
+
+static struct gkick_humanizer_params* get_humanizer_params(struct geonkick *dsp)
+{
+        size_t index = 0;
+        enum geonkick_error res = geonkick_get_current_instrument(dsp, &index);
+        if (res != GEONKICK_OK)
+                return NULL;
+
+        struct gkick_audio_output *instrument = dsp->audio->audio_outputs[index];
+        struct gkick_humanizer_params *humanizer_params = &instrument->humanizer_params;
+        return &instrument->humanizer_params;
+}
+
+static const struct gkick_humanizer_params*
+get_humanizer_params_const(const struct geonkick *dsp)
+{
+        size_t index = 0;
+        enum geonkick_error res = geonkick_get_current_instrument(dsp, &index);
+        if (res != GEONKICK_OK)
+                return NULL;
+
+        const struct gkick_audio_output *instrument = dsp->audio->audio_outputs[index];
+        const struct gkick_humanizer_params *humanizer_params = &instrument->humanizer_params;
+        return &instrument->humanizer_params;
+}
+
+enum geonkick_error
+geonkick_humanizer_enable(struct geonkick *dsp, bool b)
+{
+        struct gkick_humanizer_params *params = get_humanizer_params(dsp);
+        if (params == NULL)
+                return GEONKICK_ERROR;
+
+        params->enabled = b;
+        return GEONKICK_OK;
+}
+
+bool geonkick_humanizer_is_enabled(const struct geonkick *dsp)
+{
+        const struct gkick_humanizer_params *params = get_humanizer_params_const(dsp);
+        if (params == NULL)
+                return false;
+
+        return params->enabled;
+}
+
+enum geonkick_error
+geonkick_humanizer_set_velocity(struct geonkick *dsp, float value)
+{
+        struct gkick_humanizer_params *params =  get_humanizer_params(dsp);
+        if (params == NULL)
+                return GEONKICK_ERROR;
+
+        params->velocity = value;
+        return GEONKICK_OK;
+}
+
+float geonkick_humanizer_get_velocity(const struct geonkick *dsp)
+{
+        const struct gkick_humanizer_params *params = get_humanizer_params_const(dsp);
+        if (params == NULL)
+                return false;
+
+        return params->velocity;
+}
+
+enum geonkick_error
+geonkick_humanizer_set_timing(struct geonkick *dsp, float value)
+{
+        struct gkick_humanizer_params *params =  get_humanizer_params(dsp);
+        if (params == NULL)
+                return GEONKICK_ERROR;
+
+        params->timing = value;
+        return GEONKICK_OK;
+}
+
+float geonkick_humanizer_get_timing(const struct geonkick *dsp)
+{
+        const struct gkick_humanizer_params *params = get_humanizer_params_const(dsp);
+        if (params == NULL)
+                return false;
+
+        return params->timing;
+}
+
+void geonkick_wait_playing_ready()
+{
+        geonkick_worker_sync();
+}
