@@ -1,6 +1,6 @@
 /**
  * File name: geonkick.h
- * Project: GeonKick (A kick synthesizer)
+ * Project: GeonKick (A percussive synthesizer)
  *
  * Copyright (C) 2017 Iurie Nistor
  *
@@ -57,13 +57,16 @@ typedef float gkick_real;
 #define GEONKICK_UNUSED(expr) (void)expr
 
 #define GEONKICK_VERSION GEONKICK_VERSION_HEX
+#define GEONKICK_VERSION_MAJOR ((GEONKICK_VERSION_HEX >> 16) & 0xFF)
+#define GEONKICK_VERSION_MINOR ((GEONKICK_VERSION_HEX >> 8) & 0xFF)
+#define GEONKICK_VERSION_PATCH (GEONKICK_VERSION_HEX & 0xFF)
 #define GEONKICK_NAME "Geonkick"
 #define GEONKICK_APP_NAME "geonkick"
 #define GEOKICK_VERSION_STRING GEONKICK_VERSION_STR
 
-#if defined(GEONKICK_LIMITED_VERSION) && !defined(GEONKICK_SINGLE)
+#if defined(GEONKICK_BASIC_VERSION) && !defined(GEONKICK_SINGLE)
 #define GEONKICK_SINGLE
-#endif // GEONKICK_LIMITED_VERSION
+#endif // GEONKICK_BASIC_VERSION
 
 #ifdef GEONKICK_AUDIO_JACK
 #define GKICK_IS_STANDALONE (1)
@@ -119,11 +122,11 @@ enum GEONKICK_MODULE {
         GEONKICK_MODULE_JACK    = 1
 };
 
-#ifdef GEONKICK_LIMITED_VERSION
+#ifdef GEONKICK_BASIC_VERSION
 #define GKICK_OSC_GROUPS_NUMBER 1
 #else
 #define GKICK_OSC_GROUPS_NUMBER 3
-#endif // GEONKICK_LIMITED_VERSION
+#endif // GEONKICK_BASIC_VERSION
 #define GKICK_OSC_GROUP_SIZE 3
 
 enum geonkick_channel_type {
@@ -138,6 +141,29 @@ enum gkick_envelope_apply_type {
 	GEONKICK_ENVELOPE_APPLY_LOGARITHMIC  = 1
 };
 
+enum gkick_envelope_curve_type {
+	GEONKICK_ENVELOPE_CURVE_LINEAR  = 0,
+	GEONKICK_ENVELOPE_CURVE_BEZIER  = 1
+};
+
+enum gkick_distortion_type {
+        GEONKICK_DISTORTION_HARD_CLIPPING       = 0,
+        GEONKICK_DISTORTION_SOFT_CLIPPING_TANH  = 1,
+        GEONKICK_DISTORTION_ARCTAN              = 2,
+        GEONKICK_DISTORTION_EXPONENTIAL         = 3,
+        GEONKICK_DISTORTION_POLYNOMIAL          = 4,
+        GEONKICK_DISTORTION_LOGARITHMIC         = 5,
+        GEONKICK_DISTORTION_FOLDBACK            = 6,
+        GEONKICK_DISTORTION_HALF_WAVE_RECT      = 7,
+        GEONKICK_DISTORTION_FULL_WAVE_RECT      = 8,
+        /* Backward compatible with old presets.
+           In the next major release this to be removed. */
+#if (GEONKICK_VERSION_MAJOR > 3)
+#warning Remove backward compatibility.
+#endif // GEONKICK_VERSION_MAJOR
+        GEONKICK_DISTORTION_BACKWARD_COMPATIBLE = 9
+};
+
 #ifdef GEONKICK_SINGLE
 #define GEONKICK_MAX_INSTRUMENTS 1
 #else
@@ -153,6 +179,12 @@ enum gkick_envelope_apply_type {
 
 struct geonkick;
 
+struct gkick_envelope_point_info {
+        gkick_real x;
+        gkick_real y;
+        bool control_point;
+};
+
 enum geonkick_error
 geonkick_create(struct geonkick **kick, int sample_rate);
 
@@ -188,22 +220,21 @@ enum geonkick_error
 geonkick_osc_envelope_get_points(struct geonkick *kick,
 				 size_t osc_index,
 				 size_t env_index,
-				 gkick_real **buf,
+				 struct gkick_envelope_point_info **buf,
 				 size_t *npoints);
 
 enum geonkick_error
 geonkick_osc_envelope_set_points(struct geonkick *kick,
                                  size_t osc_index,
                                  size_t env_index,
-                                 const gkick_real *buff,
+                                 const struct gkick_envelope_point_info *buff,
                                  size_t npoints);
 
 enum geonkick_error
 geonkick_osc_envelope_add_point(struct geonkick *kick,
 				size_t osc_index,
 				size_t env_index,
-				gkick_real x,
-				gkick_real y);
+                                struct gkick_envelope_point_info *point_info);
 
 enum geonkick_error
 geonkick_osc_envelope_remove_point(struct geonkick *kick,
@@ -215,11 +246,10 @@ geonkick_osc_envelope_update_point(struct geonkick *kick,
 				   size_t osc_index,
 				   size_t env_index,
   				   size_t index,
-				   gkick_real x,
-				   gkick_real y);
+                                   struct gkick_envelope_point_info *point_info);
 
 enum geonkick_error
-geonkick_osc_envelope_set_apply_type(struct geonkick *kick, 
+geonkick_osc_envelope_set_apply_type(struct geonkick *kick,
 				     size_t osc_index,
 				     size_t env_index,
 				     enum gkick_envelope_apply_type apply_type);
@@ -235,6 +265,11 @@ geonkick_osc_set_fm(struct geonkick *kick,
                     size_t index,
                     bool is_fm);
 
+/* Used only for compatibility of older presets. */
+enum geonkick_error
+geonkick_osc_set_fm_k(struct geonkick *kick, size_t index,
+                      gkick_real k);
+
 enum geonkick_error
 geonkick_osc_is_fm(struct geonkick *kick,
                    size_t index,
@@ -325,19 +360,19 @@ geonkick_get_kick_filter_type(struct geonkick *kick,
 enum geonkick_error
 geonkick_kick_envelope_get_points(struct geonkick *kick,
                                   enum geonkick_envelope_type env_type,
-                                  gkick_real **buf,
+                                  struct gkick_envelope_point_info **buf,
                                   size_t *npoints);
 
 enum geonkick_error
 geonkick_kick_envelope_set_points(struct geonkick *kick,
                                   enum geonkick_envelope_type env_type,
-                                  const gkick_real *buff,
+                                  const struct gkick_envelope_point_info *buff,
                                   size_t npoints);
 
 enum geonkick_error
 geonkick_kick_add_env_point(struct geonkick *kick,
                             enum geonkick_envelope_type env_type,
-                            gkick_real x, gkick_real y);
+                            const struct gkick_envelope_point_info *point_info);
 
 enum geonkick_error
 geonkick_kick_remove_env_point(struct geonkick *kick,
@@ -348,11 +383,10 @@ enum geonkick_error
 geonkick_kick_update_env_point(struct geonkick *kick,
                                enum geonkick_envelope_type env_type,
                                size_t index,
-                               gkick_real x,
-                               gkick_real y);
+                               const struct gkick_envelope_point_info *point_info);
 
 enum geonkick_error
-geonkick_kick_env_set_apply_type(struct geonkick *kick, 
+geonkick_kick_env_set_apply_type(struct geonkick *kick,
 				 enum geonkick_envelope_type env_type,
 				 enum gkick_envelope_apply_type apply_type);
 
@@ -498,11 +532,19 @@ geonkick_audio_process(struct geonkick *kick,
 
 enum geonkick_error
 geonkick_distortion_enable(struct geonkick *kick,
-                           int enable);
+                           bool enable);
 
 enum geonkick_error
 geonkick_distortion_is_enabled(struct geonkick *kick,
-                               int *enabled);
+                               bool *enabled);
+
+enum geonkick_error
+geonkick_distortion_set_type(struct geonkick *kick,
+                             enum gkick_distortion_type type);
+
+enum geonkick_error
+geonkick_distortion_get_type(struct geonkick *kick,
+                             enum gkick_distortion_type *type);
 
 enum geonkick_error
 geonkick_distortion_set_in_limiter(struct geonkick *kick,
@@ -513,12 +555,12 @@ geonkick_distortion_get_in_limiter(struct geonkick *kick,
                                    gkick_real *limit);
 
 enum geonkick_error
-geonkick_distortion_set_volume(struct geonkick *kick,
-                               gkick_real volume);
+geonkick_distortion_set_out_limiter(struct geonkick *kick,
+                                    gkick_real value);
 
 enum geonkick_error
-geonkick_distortion_get_volume(struct geonkick *kick,
-                               gkick_real *volume);
+geonkick_distortion_get_out_limiter(struct geonkick *kick,
+                                    gkick_real *volume);
 
 enum geonkick_error
 geonkick_distortion_set_drive(struct geonkick *kick,
@@ -528,6 +570,56 @@ enum geonkick_error
 geonkick_distortion_get_drive(struct geonkick *kick,
                               gkick_real *drive);
 
+enum geonkick_error
+geonkick_osc_distortion_enable(struct geonkick *kick,
+                               size_t index,
+                               bool enable);
+
+enum geonkick_error
+geonkick_osc_distortion_is_enabled(struct geonkick *kick,
+                                   size_t index,
+                                   bool *enabled);
+
+enum geonkick_error
+geonkick_osc_distortion_set_type(struct geonkick *kick,
+                                 size_t index,
+                                 enum gkick_distortion_type type);
+
+enum geonkick_error
+geonkick_osc_distortion_get_type(struct geonkick *kick,
+                                 size_t index,
+                                 enum gkick_distortion_type *type);
+
+enum geonkick_error
+geonkick_osc_distortion_set_in_limiter(struct geonkick *kick,
+                                       size_t index,
+                                       gkick_real value);
+
+enum geonkick_error
+geonkick_osc_distortion_get_in_limiter(struct geonkick *kick,
+                                       size_t index,
+                                       gkick_real *value);
+
+enum geonkick_error
+geonkick_osc_distortion_set_out_limiter(struct geonkick *kick,
+                                        size_t index,
+                                        gkick_real value);
+
+enum geonkick_error
+geonkick_osc_distortion_get_out_limiter(struct geonkick *kick,
+                                        size_t index,
+                                        gkick_real *value);
+
+enum geonkick_error
+geonkick_osc_distortion_set_drive(struct geonkick *kick,
+                                  size_t index,
+                                  gkick_real drive);
+
+enum geonkick_error
+geonkick_osc_distortion_get_drive(struct geonkick *kick,
+                                  size_t index,
+                                  gkick_real *drive);
+
 int geonkick_is_module_enabed(struct geonkick *kick,
                               enum GEONKICK_MODULE module);
 
@@ -574,24 +666,24 @@ geonkick_get_osc_sample(struct geonkick *kick,
                         size_t *size);
 
 enum geonkick_error
-geonkick_set_current_percussion(struct geonkick *kick,
+geonkick_set_current_instrument(struct geonkick *kick,
                                 size_t index);
 
 enum geonkick_error
-geonkick_get_current_percussion(struct geonkick *kick,
+geonkick_get_current_instrument(const struct geonkick *kick,
                                 size_t *index);
 
 enum geonkick_error
-geonkick_unused_percussion(struct geonkick *kick,
+geonkick_unused_instrument(struct geonkick *kick,
                            int *index);
 
 enum geonkick_error
-geonkick_enable_percussion(struct geonkick *kick,
+geonkick_enable_instrument(struct geonkick *kick,
                            size_t index,
                            bool enable);
 
 enum geonkick_error
-geonkick_is_percussion_enabled(struct geonkick *kick,
+geonkick_is_instrument_enabled(struct geonkick *kick,
                                size_t index,
                                bool *enable);
 
@@ -626,16 +718,16 @@ geonkick_ged_forced_midi_channel(struct geonkick *kick,
                                  bool *force);
 
 size_t
-geonkick_midi_channels_number();
+geonkick_midi_channels_number(void);
 
 enum geonkick_error
-geonkick_set_percussion_name(struct geonkick *kick,
+geonkick_set_instrument_name(struct geonkick *kick,
                              size_t id,
                              const char *name,
                              size_t size);
 
 enum geonkick_error
-geonkick_get_percussion_name(struct geonkick *kick,
+geonkick_get_instrument_name(struct geonkick *kick,
                              size_t id,
                              char *name,
                              size_t size);
@@ -648,42 +740,42 @@ size_t
 geonkick_channels_number();
 
 enum geonkick_error
-geonkick_set_percussion_channel(struct geonkick *kick,
+geonkick_set_instrument_channel(struct geonkick *kick,
                                 size_t id,
                                 size_t channel);
 
 enum geonkick_error
-geonkick_get_percussion_channel(struct geonkick *kick,
+geonkick_get_instrument_channel(struct geonkick *kick,
                                 size_t id,
                                 size_t *channel);
 
 enum geonkick_error
-geonkick_percussion_set_limiter(struct geonkick *kick,
+geonkick_instrument_set_limiter(struct geonkick *kick,
                                 size_t id,
                                 gkick_real val);
 
 enum geonkick_error
-geonkick_percussion_get_limiter(struct geonkick *kick,
+geonkick_instrument_get_limiter(struct geonkick *kick,
                                 size_t id,
                                 gkick_real *val);
 
 enum geonkick_error
-geonkick_percussion_mute(struct geonkick *kick,
+geonkick_instrument_mute(struct geonkick *kick,
                          size_t id,
                          bool b);
 
 enum geonkick_error
-geonkick_percussion_is_muted(struct geonkick *kick,
+geonkick_instrument_is_muted(struct geonkick *kick,
                              size_t id,
                              bool *b);
 
 enum geonkick_error
-geonkick_percussion_solo(struct geonkick *kick,
+geonkick_instrument_solo(struct geonkick *kick,
                          size_t id,
                          bool b);
 
 enum geonkick_error
-geonkick_percussion_is_solo(struct geonkick *kick,
+geonkick_instrument_is_solo(struct geonkick *kick,
                             size_t id,
                             bool *b);
 
@@ -703,12 +795,12 @@ enum geonkick_error
 geonkick_get_sample_preview_limiter(struct geonkick *kick, gkick_real *val);
 
 enum geonkick_error
-geonkick_percussion_enable_note_off(struct geonkick *kick,
+geonkick_instrument_enable_note_off(struct geonkick *kick,
                                     size_t id,
                                     bool enable);
 
 enum geonkick_error
-geonkick_percussion_note_off_enabled(struct geonkick *kick,
+geonkick_instrument_note_off_enabled(struct geonkick *kick,
                                      size_t id,
                                      bool *enabled);
 
@@ -719,6 +811,17 @@ geonkick_note_off_enabled(struct geonkick *kick,
 
 size_t geonkick_layers_number(void);
 
+enum geonkick_error
+geonkick_humanizer_enable(struct geonkick *dsp, bool b);
+bool geonkick_humanizer_is_enabled(const struct geonkick *dsp);
+enum geonkick_error
+geonkick_humanizer_set_velocity(struct geonkick *dsp, float value);
+float geonkick_humanizer_get_velocity(const struct geonkick *dsp);
+enum geonkick_error
+geonkick_humanizer_set_timing(struct geonkick *dsp, float value);
+float geonkick_humanizer_get_timing(const struct geonkick *dsp);
+void geonkick_wait_playing_ready();
+
 bool
 geonkick_is_plugin();
 
