@@ -1,6 +1,6 @@
 /**
  * File name: oscillator.c
- * Project: Geonkick (A kick synthesizer)
+ * Project: Geonkick (A percussive synthesizer)
  *
  * Copyright (C) 2017 Iurie Nistor
  *
@@ -41,10 +41,11 @@ enum geonkick_osc_state {
 };
 
 enum gkick_osc_envelope_type {
-        GKICK_OSC_AMPLITUDE_ENVELOPE     = 0,
-        GKICK_OSC_FREQUENCY_ENVELOPE     = 1,
-        GKICK_OSC_PITCH_SHIFT_ENVELOPE   = 2,
-        GKICK_OSC_NOISE_DENSITY_ENVELOPE = 3
+        GKICK_OSC_AMPLITUDE_ENVELOPE        = 0,
+        GKICK_OSC_FREQUENCY_ENVELOPE        = 1,
+        GKICK_OSC_PITCH_SHIFT_ENVELOPE      = 2,
+        GKICK_OSC_NOISE_DENSITY_ENVELOPE    = 3,
+        GKICK_OSC_DISTORTION_DRIVE_ENVELOPE = 4
 };
 
 struct gkick_oscillator {
@@ -70,14 +71,18 @@ struct gkick_oscillator {
 
         /* FM input value for this OSC. */
         gkick_real fm_input;
+        /* FM input coefficient. Used for compatibity of presets. */
+        gkick_real fm_k;
 
         /* Specifies if this OSC is a FM source to other oscillator. */
         bool is_fm;
 
 	size_t env_number;
 	struct gkick_envelope **envelopes;
+        bool filter_enabled;
         struct gkick_filter *filter;
-        int filter_enabled;
+        bool distortion_enabled;
+        struct gkick_distortion *distortion;
 	pthread_mutex_t lock;
 };
 
@@ -136,12 +141,12 @@ gkick_osc_func_sample(struct gkick_buffer *sample,
 
 void gkick_osc_get_envelope_points(struct gkick_oscillator *osc,
                                    size_t env_index,
-                                   gkick_real **buff,
+                                   struct gkick_envelope_point_info **buff,
                                    size_t *npoints);
 
 void gkick_osc_set_envelope_points(struct gkick_oscillator *osc,
                                    size_t env_index,
-                                   const gkick_real *buff,
+                                   const struct gkick_envelope_point_info *buff,
                                    size_t npoints);
 
 unsigned int gkick_osc_get_noise_density(struct gkick_oscillator *osc,
