@@ -69,8 +69,10 @@ gkick_synth_new(struct gkick_synth **synth, int sample_rate)
                 return GEONKICK_ERROR;
         } else {
                 /* Add two default points. */
-                gkick_envelope_add_point((*synth)->envelope, 0.0f, 1.0f);
-                gkick_envelope_add_point((*synth)->envelope, 1.0f, 1.0f);
+                struct gkick_envelope_point_info info = {0.0f, 1.0f, false};
+                gkick_envelope_add_point((*synth)->envelope, &info);
+                info.x = 1.0f;
+                gkick_envelope_add_point((*synth)->envelope, &info);
         }
 
         /* Create synthesizer kick buffer. */
@@ -313,7 +315,7 @@ enum geonkick_error
 gkick_synth_osc_envelope_points(struct gkick_synth *synth,
                                 int osc_index,
                                 int env_index,
-                                gkick_real **buf,
+                                struct gkick_envelope_point_info **buf,
                                 size_t *npoints)
 {
         if (synth == NULL || buf == NULL || npoints == NULL) {
@@ -341,7 +343,7 @@ enum geonkick_error
 gkick_synth_osc_envelope_set_points(struct gkick_synth *synth,
                                     int osc_index,
                                     int env_index,
-                                    const gkick_real *buf,
+                                    const struct gkick_envelope_point_info *buf,
                                     size_t npoints)
 {
         if (synth == NULL || buf == NULL || npoints == 0) {
@@ -370,8 +372,7 @@ enum geonkick_error
 gkick_synth_osc_env_add_point(struct gkick_synth *synth,
                               int osc_index,
                               int env_index,
-                              gkick_real x,
-                              gkick_real y)
+                              struct gkick_envelope_point_info *point_info)
 {
         if (synth == NULL) {
                 gkick_log_error("wrong arguments");
@@ -391,7 +392,7 @@ gkick_synth_osc_env_add_point(struct gkick_synth *synth,
                 gkick_log_error("can't get envelope");
                 return GEONKICK_ERROR;
         }
-        if (gkick_envelope_add_point(env, x, y) == NULL) {
+        if (gkick_envelope_add_point(env, point_info) == NULL) {
                 gkick_log_error("can't add envelope point");
                 gkick_synth_unlock(synth);
                 return GEONKICK_ERROR;
@@ -445,8 +446,7 @@ gkick_synth_osc_env_update_point(struct gkick_synth *synth,
                                  int osc_index,
                                  int env_index,
                                  int index,
-                                 gkick_real x,
-                                 gkick_real y)
+                                 struct gkick_envelope_point_info *point_info)
 {
         if (synth == NULL) {
                 gkick_log_error("wrong arguments");
@@ -468,7 +468,7 @@ gkick_synth_osc_env_update_point(struct gkick_synth *synth,
                 return GEONKICK_ERROR;
         }
 
-        gkick_envelope_update_point(env, index, x, y);
+        gkick_envelope_update_point(env, index, point_info);
         if (synth->osc_groups[osc_index / GKICK_OSC_GROUP_SIZE]
             && osc->state == GEONKICK_OSC_STATE_ENABLED) {
                 synth->buffer_update = true;
@@ -912,7 +912,7 @@ synth_get_kick_envelope(struct gkick_synth *synth,
 enum geonkick_error
 gkick_synth_kick_envelope_get_points(struct gkick_synth *synth,
                                      enum geonkick_envelope_type env_type,
-                                     gkick_real **buf,
+                                     struct gkick_envelope_point_info **buf,
                                      size_t *npoints)
 {
         if (synth == NULL || buf == NULL) {
@@ -933,7 +933,7 @@ gkick_synth_kick_envelope_get_points(struct gkick_synth *synth,
 enum geonkick_error
 gkick_synth_kick_envelope_set_points(struct gkick_synth *synth,
                                      enum geonkick_envelope_type env_type,
-                                     const gkick_real *buf,
+                                     const struct gkick_envelope_point_info *buf,
                                      size_t npoints)
 {
         if (synth == NULL || buf == NULL) {
@@ -962,8 +962,7 @@ gkick_synth_kick_envelope_set_points(struct gkick_synth *synth,
 enum geonkick_error
 gkick_synth_kick_add_env_point(struct gkick_synth *synth,
                                enum geonkick_envelope_type env_type,
-                               gkick_real x,
-                               gkick_real y)
+                               const struct gkick_envelope_point_info *point_info)
 {
         if (synth == NULL) {
                 gkick_log_error("wrong arguments");
@@ -973,7 +972,7 @@ gkick_synth_kick_add_env_point(struct gkick_synth *synth,
         gkick_synth_lock(synth);
 	struct gkick_envelope *env = synth_get_kick_envelope(synth, env_type);
 	if (env != NULL)
-		gkick_envelope_add_point(env, x, y);
+		gkick_envelope_add_point(env, point_info);
 
         if (env_type == GEONKICK_AMPLITUDE_ENVELOPE
             || ((env_type == GEONKICK_FILTER_CUTOFF_ENVELOPE
@@ -1019,8 +1018,7 @@ enum geonkick_error
 gkick_synth_kick_update_env_point(struct gkick_synth *synth,
                                   enum geonkick_envelope_type env_type,
                                   size_t index,
-                                  gkick_real x,
-                                  gkick_real y)
+                                  const struct gkick_envelope_point_info *point_info)
 {
         if (synth == NULL) {
                 gkick_log_error("wrong arguments");
@@ -1030,7 +1028,7 @@ gkick_synth_kick_update_env_point(struct gkick_synth *synth,
         gkick_synth_lock(synth);
 	struct gkick_envelope *env = synth_get_kick_envelope(synth, env_type);
 	if (env != NULL)
-		gkick_envelope_update_point(env, index, x, y);
+		gkick_envelope_update_point(env, index, point_info);
 
         if (env_type == GEONKICK_AMPLITUDE_ENVELOPE
             || ((env_type == GEONKICK_FILTER_CUTOFF_ENVELOPE
@@ -1349,7 +1347,7 @@ gkick_synth_process(struct gkick_synth *synth)
 	gkick_filter_init(synth->filter);
 	gkick_synth_unlock(synth);
 
-	/* Synthesize the percussion into the synthesizer buffer. */
+	/* Synthesize the instrument into the synthesizer buffer. */
 	size_t i = 0;
         size_t tries = 0;
 	while (1) {
@@ -1445,7 +1443,7 @@ gkick_synth_get_value(struct gkick_synth *synth,
         if (synth->filter_enabled)
                 gkick_filter_val(synth->filter, val, &val, env_x);
 
-        int enabled = 0;
+        bool enabled = false;
         gkick_distortion_is_enabled(synth->distortion, &enabled);
         if (enabled)
                 gkick_distortion_val(synth->distortion, val, &val, env_x);
@@ -1701,7 +1699,7 @@ gkick_synth_osc_is_enabled_filter(struct gkick_synth *synth,
 
 enum geonkick_error
 gkick_synth_distortion_enable(struct gkick_synth *synth,
-                              int enable)
+                              bool enable)
 {
 	synth->buffer_update = true;
         return gkick_distortion_enable(synth->distortion,
@@ -1710,40 +1708,63 @@ gkick_synth_distortion_enable(struct gkick_synth *synth,
 
 enum geonkick_error
 gkick_synth_distortion_is_enabled(struct gkick_synth *synth,
-                                  int *enabled)
+                                  bool *enabled)
 {
         return gkick_distortion_is_enabled(synth->distortion,
                                            enabled);
 }
 
 enum geonkick_error
-gkick_synth_distortion_set_in_limiter(struct gkick_synth *synth,
-                                      gkick_real limit)
+gkick_synth_distortion_set_type(struct gkick_synth *synth,
+                                enum gkick_distortion_type type)
 {
-        gkick_distortion_set_in_limiter(synth->distortion,
-                                              limit);
-	int enabled = 0;
-        gkick_distortion_is_enabled(synth->distortion,
-                                          &enabled);
-        if (enabled)
+        enum geonkick_error res;
+        gkick_distortion_set_type(synth->distortion, type);
+	bool enabled = false;
+        res = gkick_distortion_is_enabled(synth->distortion,
+                                               &enabled);
+        if (res == GEONKICK_OK && enabled)
                 synth->buffer_update = true;
-        return GEONKICK_OK;
+        return res;
+}
+
+enum geonkick_error
+gkick_synth_distortion_get_type(struct gkick_synth *synth,
+                                enum gkick_distortion_type *type)
+{
+        return gkick_distortion_get_type(synth->distortion,
+                                         type);
+}
+
+enum geonkick_error
+gkick_synth_distortion_set_in_limiter(struct gkick_synth *synth,
+                                      gkick_real value)
+{
+        enum geonkick_error res;
+        gkick_distortion_set_in_limiter(synth->distortion,
+                                        value);
+	bool enabled = false;
+        res = gkick_distortion_is_enabled(synth->distortion,
+                                          &enabled);
+        if (res == GEONKICK_OK && enabled)
+                synth->buffer_update = true;
+        return res;
 }
 
 enum geonkick_error
 gkick_synth_distortion_get_in_limiter(struct gkick_synth *synth,
-                                      gkick_real *limit)
+                                      gkick_real *value)
 {
 	return gkick_distortion_get_in_limiter(synth->distortion,
-                                               limit);
+                                               value);
 }
 
 enum geonkick_error
-gkick_synth_distortion_set_volume(struct gkick_synth *synth, gkick_real volume)
+gkick_synth_distortion_set_out_limiter(struct gkick_synth *synth, gkick_real value)
 {
         enum geonkick_error res;
-        int enabled;
-        res = gkick_distortion_set_volume(synth->distortion, volume);
+        bool enabled = false;
+        res = gkick_distortion_set_out_limiter(synth->distortion, value);
         gkick_distortion_is_enabled(synth->distortion, &enabled);
         if (res == GEONKICK_OK && enabled)
                 synth->buffer_update = true;
@@ -1751,11 +1772,11 @@ gkick_synth_distortion_set_volume(struct gkick_synth *synth, gkick_real volume)
 }
 
 enum geonkick_error
-gkick_synth_distortion_get_volume(struct gkick_synth *synth,
-                                  gkick_real *volume)
+gkick_synth_distortion_get_out_limiter(struct gkick_synth *synth,
+                                       gkick_real *value)
 {
-        return gkick_distortion_get_volume(synth->distortion,
-                                           volume);
+        return gkick_distortion_get_out_limiter(synth->distortion,
+                                                value);
 }
 
 enum geonkick_error
@@ -1763,7 +1784,7 @@ gkick_synth_distortion_set_drive(struct gkick_synth *synth,
                                  gkick_real drive)
 {
         enum geonkick_error res;
-        int enabled;
+        bool enabled = false;
         res = gkick_distortion_set_drive(synth->distortion, drive);
         gkick_distortion_is_enabled(synth->distortion, &enabled);
         if (res == GEONKICK_OK && enabled)
