@@ -1,6 +1,6 @@
 /**
  * File name: envelope.h
- * Project: Geonkick (A kick synthesizer)
+ * Project: Geonkick (A percussive synthesizer)
  *
  * Copyright (C) 2017 Iurie Nistor
  *
@@ -386,17 +386,40 @@ void Envelope::drawLines(RkPainter &painter)
         if (envelopePoints.size() < 2)
                 return;
 
-        std::vector<RkPoint> points;
-	for (const auto& point : envelopePoints) {
-                auto scaledPoint = scaleUp(point);
-	        points.push_back(RkPoint(scaledPoint.x(), -scaledPoint.y()));
-	}
-
-	auto pen = painter.pen();
-	pen.setWidth(2);
+        auto pen = painter.pen();
+        pen.setWidth(2);
         pen.setColor(RkColor(200, 200, 200, 150));
-	painter.setPen(pen);
-	painter.drawPolyline(points);
+        painter.setPen(pen);
+
+        size_t i = 0;
+        while (i < envelopePoints.size() - 1) {
+                if (i + 1 < envelopePoints.size() && !envelopePoints[i + 1].isControlPoint()) {
+                        std::vector<RkPoint> points;
+                        auto scaledPoint1 = scaleUp(envelopePoints[i]);
+                        points.push_back(RkPoint(scaledPoint1.x(), -scaledPoint1.y()));
+
+                        auto scaledPoint2 = scaleUp(envelopePoints[i + 1]);
+                        points.push_back(RkPoint(scaledPoint2.x(), -scaledPoint2.y()));
+
+                        painter.drawPolyline(points);
+                        i += 1;
+                } else if (i + 2 < envelopePoints.size()) {
+                        std::vector<RkPoint> points;
+                        auto scaledPoint1 = scaleUp(envelopePoints[i]);
+                        points.push_back(RkPoint(scaledPoint1.x(), -scaledPoint1.y()));
+
+                        auto scaledPoint2 = scaleUp(envelopePoints[i + 1]);
+                        points.push_back(RkPoint(scaledPoint2.x(), -scaledPoint2.y()));
+
+                        auto scaledPoint3 = scaleUp(envelopePoints[i + 2]);
+                        points.push_back(RkPoint(scaledPoint3.x(), -scaledPoint3.y()));
+
+                        painter.drawCurve(points);
+                        i += 2;
+                } else {
+                        break;
+                }
+        }
 }
 
 void Envelope::overPoint(const RkPoint &point)
@@ -438,9 +461,9 @@ void Envelope::selectPoint(const RkPoint &point)
 void Envelope::updateSelectedPointValue(double val)
 {
         if (hasEditingPoint() && editedPointIndex < envelopePoints.size()) {
-                pointUpdatedEvent(editedPointIndex,
-                                  envelopePoints[editedPointIndex].x(),
-                                  convertFromHumanValue(val));
+                EnvelopePoint p = envelopePoints[editedPointIndex];
+                p.setY(convertFromHumanValue(val));
+                pointUpdatedEvent(editedPointIndex, p);
                 updatePoints();
         }
 }
@@ -461,7 +484,7 @@ double Envelope::getSelectedPointValue() const
         return convertToHumanValue(getSelectedPoint().y());
 }
 
-RkRealPoint Envelope::getSelectedPoint() const
+EnvelopePoint Envelope::getSelectedPoint() const
 {
         if (hasSelected()) {
                 if (overPointIndex < envelopePoints.size())
@@ -516,57 +539,72 @@ void Envelope::moveSelectedPoint(int x, int y)
 	else
                 selectedPoint.setX(scaledPoint.x());
 	selectedPoint.setY(scaledPoint.y());
-        pointUpdatedEvent(selectedPointIndex, selectedPoint.x(), selectedPoint.y());
+        pointUpdatedEvent(selectedPointIndex, selectedPoint);
 }
 
-void Envelope::setPoints(const std::vector<RkRealPoint> &points)
+void Envelope::setPoints(const std::vector<EnvelopePoint> &points)
 {
         removePoints();
         for (const auto &point : points)
                 envelopePoints.push_back(point);
 }
 
-void Envelope::addPoint(const RkPoint &point)
+void Envelope::addPoint(const RkPoint &point, bool isControlPoint)
 {
-        auto scaledPoint = scaleDown(point);
-        if (scaledPoint.y() < 0)
-                scaledPoint.setY(0);
-        else if (scaledPoint.y() > 1)
-                scaledPoint.setY(1);
+        EnvelopePoint scaledPoint = scaleDown(point);
+        scaledPoint.setAsControlPoint(isControlPoint);
 
-  	if (scaledPoint.x() > 1) {
-	        scaledPoint.setX(1);
-		envelopePoints.push_back(scaledPoint);
-	} else if (scaledPoint.x() < 0) {
-                scaledPoint.setX(0);
-                envelopePoints.insert(envelopePoints.begin(), scaledPoint);
-        } else if (envelopePoints.empty()) {
-                envelopePoints.push_back(scaledPoint);
-	} else if (scaledPoint.x() <= envelopePoints[0].x()) {
-                envelopePoints.insert(envelopePoints.begin(), scaledPoint);
-	} else if (scaledPoint.x() >= envelopePoints.back().x()) {
-                envelopePoints.push_back(scaledPoint);
-	} else {
-		for (auto it = envelopePoints.begin(); it != envelopePoints.end(); ++it) {
-			if (scaledPoint.x() <= it->x()) {
-                                envelopePoints.insert(it, scaledPoint);
-                                break;
-			}
-		}
-	}
-        pointAddedEvent(scaledPoint.x(), scaledPoint.y());
+        scaledPoint.setY(std::clamp(scaledPoint.y(), 0.0, 1.0));
+        scaledPoint.setX(std::clamp(scaledPoint.x(), 0.0, 1.0));
+
+        auto it = std::lower_bound(
+                                   envelopePoints.begin(), envelopePoints.end(), scaledPoint.x(),
+                                   [](const EnvelopePoint &ep, float xVal) {
+                                           return ep.x() < xVal;
+                                   });
+
+        if (isControlPoint) {
+                if (it == envelopePoints.begin() || it == envelopePoints.end())
+                        return;
+                auto prev = std::prev(it);
+                auto next = it;
+                if (prev->isControlPoint() || next->isControlPoint())
+                        scaledPoint.setAsControlPoint(false);
+        }
+
+        envelopePoints.insert(it, scaledPoint);
+        pointAddedEvent(scaledPoint);
 }
 
 void Envelope::removePoint(const RkPoint &point)
 {
-        for (decltype(envelopePoints.size()) i = 0; i < envelopePoints.size(); i++) {
-		if (hasPoint(envelopePoints[i], point)) {
-			if (i != 0 && i != envelopePoints.size() - 1) {
-				envelopePoints.erase(envelopePoints.begin() + i);
-                                pointRemovedEvent(i);
-			}
-			break;
-		}
+        auto it = std::find_if(envelopePoints.begin(), envelopePoints.end(),
+                               [&](const EnvelopePoint &ep) {
+                                       return hasPoint(ep, point);
+                               });
+
+        if (it == envelopePoints.end()
+            || it == envelopePoints.begin()
+            || it == envelopePoints.end() - 1)
+                return;
+
+        auto index = std::distance(envelopePoints.begin(), it);
+        it = envelopePoints.erase(it);
+        pointRemovedEvent(index);
+
+        if (it != envelopePoints.begin() && it != envelopePoints.end() - 1) {
+                auto prev = std::prev(it);
+                auto next = it;
+                if (prev->isControlPoint() && next->isControlPoint()) {
+                        size_t nextIndex = std::distance(envelopePoints.begin(), next);
+                        size_t prevIndex = nextIndex - 1;
+
+                        envelopePoints.erase(next);
+                        pointRemovedEvent(nextIndex);
+
+                        envelopePoints.erase(prev);
+                        pointRemovedEvent(prevIndex);
+                }
         }
 }
 
