@@ -1,8 +1,8 @@
 /**
  * File name: envelope_draw_area.h
- * Project: Geonkick (A kick synthesizer)
+ * Project: Geonkick (A percussive synthesizer)
  *
- * Copyright (C) 2017 Iurie Nistor 
+ * Copyright (C) 2017 Iurie Nistor
  *
  * This file is part of Geonkick.
  *
@@ -32,18 +32,18 @@
 #include "RkImage.h"
 
 class Envelope;
-class KickGraph;
+class InstrumentWaveform;
 class RkMouseEvent;
-class KickGraph;
-class GeonkickApi;
+class InstrumentWaveform;
+class DspProxy;
 
 class EnvelopeWidgetDrawingArea : public GeonkickWidget
 {
  public:
-   EnvelopeWidgetDrawingArea(GeonkickWidget *parent, GeonkickApi *api);
+   EnvelopeWidgetDrawingArea(GeonkickWidget *parent, DspProxy *dsp);
    Envelope* getEnvelope() const;
    const RkRect getDrawingArea();
-   void updateKickGraph(const std::shared_ptr<RkImage> &graphImage);
+   void updateInstrumentWaveform(const std::shared_ptr<RkImage> &waveformImage);
    bool isHideEnvelope() const;
    void setEnvelope(Envelope* envelope);
    void envelopeUpdated();
@@ -51,6 +51,8 @@ class EnvelopeWidgetDrawingArea : public GeonkickWidget
    void setPointEditingMode(bool b = true);
    void zoomIn();
    void zoomOut();
+   void setBezierMode(bool b);
+   bool isBezierMode() const;
    RK_DECL_ACT(isOverPoint,
                isOverPoint(const std::string info),
                RK_ARG_TYPE(const std::string),
@@ -67,18 +69,21 @@ class EnvelopeWidgetDrawingArea : public GeonkickWidget
    void mouseButtonReleaseEvent(RkMouseEvent *event) override;
    void mouseDoubleClickEvent(RkMouseEvent *event) override;
    void wheelEvent(RkWheelEvent *event) override;
+   void shortcutEvent(RkKeyEvent *event) override;
    std::string getEnvStateText() const;
 
  private:
-   GeonkickApi* geonkickApi;
+   DspProxy* dspProxy;
    Envelope* currentEnvelope;
    RkRect drawingArea;
    RkPoint mousePoint;
    bool hideEnvelope;
-   std::shared_ptr<RkImage> kickGraphImage;
+   std::shared_ptr<RkImage> instrumentWaveformImage;
    RkImage envelopeImage;
-   KickGraph *kickGraphics;
+   InstrumentWaveform *instrumentWaveform;
    bool pointEditingMode;
+   bool addAsControlPoint;
+   bool bezierMode;
 };
 
 #endif // ENVELOPE_DRAW_AREA_H
