@@ -1,6 +1,6 @@
 /**
  * File name: envelope_widget.cpp
- * Project: Geonkick (A kick synthesizer)
+ * Project: Geonkick (A percussive synthesizer)
  *
  * Copyright (C) 2017 Iurie Nistor
  *
@@ -27,9 +27,12 @@
 #include "envelope_draw_area.h"
 #include "geonkick_button.h"
 #include "ViewState.h"
+#include "GeonkickModel.h"
+#include "OscillatorModel.h"
+#include "GeonkickConfig.h"
 
-#include <RkContainer.h>
-#include <RkLabel.h>
+#include "RkContainer.h"
+#include "RkLabel.h"
 
 RK_DECLARE_IMAGE_RC(separator);
 RK_DECLARE_IMAGE_RC(layer1_env);
@@ -38,53 +41,55 @@ RK_DECLARE_IMAGE_RC(layer2_env);
 RK_DECLARE_IMAGE_RC(layer2_env_active);
 RK_DECLARE_IMAGE_RC(layer3_env);
 RK_DECLARE_IMAGE_RC(layer3_env_active);
+RK_DECLARE_IMAGE_RC(bezier_mode_button);
+RK_DECLARE_IMAGE_RC(bezier_mode_button_on);
+RK_DECLARE_IMAGE_RC(bezier_mode_button_hover);
 
-EnvelopeWidget::EnvelopeWidget(GeonkickWidget *parent,
-                               GeonkickApi *api,
-                               const std::vector<std::unique_ptr<Oscillator>> &oscillators)
+EnvelopeWidget::EnvelopeWidget(GeonkickWidget *parent, GeonkickModel *model)
         : GeonkickWidget(parent)
         , drawArea{nullptr}
-#ifndef GEONKICK_LIMITED_VERSION
+#ifndef GEONKICK_BASIC_VERSION
         , layer1Button{nullptr}
         , layer2Button{nullptr}
         , layer3Button{nullptr}
-#endif // GEONKICK_LIMITED_VERSION
-        , geonkickApi{api}
+#endif // GEONKICK_BASIC_VERSION
+        , bezierModeButton {nullptr}
+        , geonkickModel{model}
+        , dspProxy{geonkickModel->getDspProxy()}
+        , oscillators{geonkickModel->getOscillatorModels()}
 {
         // Create drawing area.
-        drawArea = new EnvelopeWidgetDrawingArea(this, geonkickApi);
+        drawArea = new EnvelopeWidgetDrawingArea(this, dspProxy);
         auto rect = drawArea->getDrawingArea();
 
         // Oscillator1 envelope
-        auto oscillator = oscillators[static_cast<int>(Oscillator::Type::Oscillator1)].get();
+        auto oscillator = oscillators[static_cast<int>(OscillatorModel::Type::Oscillator1)];
         auto envelope = std::make_unique<OscillatorEnvelope>(oscillator, rect);
         envelope->setCategory(Envelope::Category::Oscillator1);
         envelopes.insert({static_cast<int>(Envelope::Category::Oscillator1),
                         std::move(envelope)});
 
         // Oscillator2 envelope
-        oscillator = oscillators[static_cast<int>(Oscillator::Type::Oscillator2)].get();
+        oscillator = oscillators[static_cast<int>(OscillatorModel::Type::Oscillator2)];
         envelope = std::make_unique<OscillatorEnvelope>(oscillator, rect);
         envelope->setCategory(Envelope::Category::Oscillator2);
         envelopes.insert({static_cast<int>(Envelope::Category::Oscillator2),
                         std::move(envelope)});
 
         // Oscillator3 envelope
-        oscillator = oscillators[static_cast<int>(Oscillator::Type::Oscillator3)].get();
+        oscillator = oscillators[static_cast<int>(OscillatorModel::Type::Oscillator3)];
         envelope = std::make_unique<OscillatorEnvelope>(oscillator, rect);
         envelope->setCategory(Envelope::Category::Oscillator3);
         envelopes.insert({static_cast<int>(Envelope::Category::Oscillator3),
                         std::move(envelope)});
 
         // General envelope
-        auto generalEnvelope = std::make_unique<GeneralEnvelope>(geonkickApi, rect);
-        generalEnvelope->setCategory(Envelope::Category::General);
-        envelopes.insert({static_cast<int>(Envelope::Category::General),
+        auto generalEnvelope = std::make_unique<GeneralEnvelope>(dspProxy, rect);
+        generalEnvelope->setCategory(Envelope::Category::InstrumentGlobal);
+        envelopes.insert({static_cast<int>(Envelope::Category::InstrumentGlobal),
                         std::move(generalEnvelope)});
-#ifndef GEONKICK_LIMITED_VERSION
         createButtomMenu();
-#endif // GEONKICK_LIMITED_VERSION
-        showEnvelope(Envelope::Category::General, Envelope::Type::Amplitude);
+        showEnvelope(Envelope::Category::InstrumentGlobal, Envelope::Type::Amplitude);
         RK_ACT_BIND(viewState(), envelopeChanged,
                     RK_ACT_ARGS(Envelope::Category category, Envelope::Type envelope),
                     this, showEnvelope(category, envelope));
@@ -94,25 +99,54 @@ EnvelopeWidget::EnvelopeWidget(GeonkickWidget *parent,
         drawArea->show();
 }
 
-#ifndef GEONKICK_LIMITED_VERSION
 void EnvelopeWidget::createButtomMenu()
 {
         auto buttomAreaWidget = new GeonkickWidget(drawArea);
         buttomAreaWidget->setBackgroundColor(40, 40, 40);
-        buttomAreaWidget->setFixedSize(90, 20);
+#ifndef GEONKICK_BASIC_VERSION
+        buttomAreaWidget->setFixedSize(120, 20);
+#else
+        buttomAreaWidget->setFixedSize(25, 20);
+#endif  // GEONKICK_BASIC_VERSION
         buttomAreaWidget->setPosition(55 + drawArea->x(),
                                       drawArea->y() + drawArea->height() - buttomAreaWidget->height() - 6);
 
         auto menuContainer = new RkContainer(buttomAreaWidget);
+#ifndef GEONKICK_BASIC_VERSION
         createLayersButtons(buttomAreaWidget);
         menuContainer->addWidget(layer1Button);
         menuContainer->addSpace(5);
         menuContainer->addWidget(layer2Button);
         menuContainer->addSpace(5);
         menuContainer->addWidget(layer3Button);
+        menuContainer->addSpace(10);
+#endif // GEONKICK_BASIC_VERSION
+        createBezierModeControls(buttomAreaWidget, menuContainer);
         buttomAreaWidget->show();
 }
-#endif // GEONKICK_LIMITED_VERSION
+
+void EnvelopeWidget::createBezierModeControls(GeonkickWidget* widget, RkContainer *container)
+{
+        bezierModeButton = new GeonkickButton(widget);
+        bezierModeButton->setSize(24, 18);
+        bezierModeButton->setCheckable(true);
+        bezierModeButton->setPressed(GeonkickConfig().isBezierMode());
+        bezierModeButton->setImage(RK_RC_IMAGE(bezier_mode_button),
+                                  RkButton::State::Unpressed);
+        bezierModeButton->setImage(RK_RC_IMAGE(bezier_mode_button_on),
+                                  RkButton::State::Pressed);
+        bezierModeButton->setImage(RK_RC_IMAGE(bezier_mode_button_hover),
+                                  RkButton::State::UnpressedHover);
+        bezierModeButton->setImage(RK_RC_IMAGE(bezier_mode_button_hover),
+                                  RkButton::State::PressedHover);
+        RK_ACT_BIND(bezierModeButton,
+                    toggled,
+                    RK_ACT_ARGS(bool pressed),
+                    this,
+                    enableBezierMode(pressed));
+
+        container->addWidget(bezierModeButton);
+}
 
 void EnvelopeWidget::createPointInfoLabel()
 {
@@ -140,10 +174,10 @@ void EnvelopeWidget::createZoomInfoLabel()
                     zoomInfoLabel, setText("1:" + info));
 }
 
-void EnvelopeWidget::updateKickGraph(std::shared_ptr<RkImage> graphImage)
+void EnvelopeWidget::updateInstrumentWaveform(std::shared_ptr<RkImage> waveformImage)
 {
-        if (graphImage && !graphImage->isNull())
-                drawArea->updateKickGraph(graphImage);
+        if (waveformImage && !waveformImage->isNull())
+                drawArea->updateInstrumentWaveform(waveformImage);
 }
 
 Envelope* EnvelopeWidget::getEnvelope(Envelope::Category category)
@@ -166,7 +200,7 @@ void EnvelopeWidget::hideEnvelope(bool b)
         drawArea->setHideEnvelope(b);
 }
 
-#ifndef GEONKICK_LIMITED_VERSION
+#ifndef GEONKICK_BASIC_VERSION
 void EnvelopeWidget::createLayersButtons(GeonkickWidget *buttomAreaWidget)
 {
         layer1Button = new GeonkickButton(buttomAreaWidget);
@@ -181,7 +215,7 @@ void EnvelopeWidget::createLayersButtons(GeonkickWidget *buttomAreaWidget)
         layer1Button->setCheckable(true);
         layer1Button->setPressed(true);
         RK_ACT_BIND(layer1Button, toggled, RK_ACT_ARGS(bool b),
-                    this, setLayer(GeonkickApi::Layer::Layer1));
+                    this, setLayer(DspProxy::Layer::Layer1));
 
         layer2Button = new GeonkickButton(buttomAreaWidget);
         layer2Button->setBackgroundColor(buttomAreaWidget->background());
@@ -194,7 +228,7 @@ void EnvelopeWidget::createLayersButtons(GeonkickWidget *buttomAreaWidget)
                                RkButton::State::UnpressedHover);
         layer2Button->setCheckable(true);
         RK_ACT_BIND(layer2Button, toggled, RK_ACT_ARGS(bool b),
-                    this, setLayer(GeonkickApi::Layer::Layer2));
+                    this, setLayer(DspProxy::Layer::Layer2));
 
         layer3Button = new GeonkickButton(buttomAreaWidget);
         layer3Button->setBackgroundColor(buttomAreaWidget->background());
@@ -207,18 +241,18 @@ void EnvelopeWidget::createLayersButtons(GeonkickWidget *buttomAreaWidget)
                                RkButton::State::UnpressedHover);
         layer3Button->setCheckable(true);
         RK_ACT_BIND(layer3Button, toggled, RK_ACT_ARGS(bool b),
-                    this, setLayer(GeonkickApi::Layer::Layer3));
+                    this, setLayer(DspProxy::Layer::Layer3));
 }
-#endif // GEONKICK_LIMITED_VERSION
+#endif // GEONKICK_BASIC_VERSION
 
-void EnvelopeWidget::setLayer(GeonkickApi::Layer layer)
+void EnvelopeWidget::setLayer(DspProxy::Layer layer)
 {
-#ifndef GEONKICK_LIMITED_VERSION
-        layer1Button->setPressed(GeonkickApi::Layer::Layer1 == layer);
-        layer2Button->setPressed(GeonkickApi::Layer::Layer2 == layer);
-        layer3Button->setPressed(GeonkickApi::Layer::Layer3 == layer);
-#endif // GEONKICK_LIMITED_VERSION
-        geonkickApi->setLayer(layer);
+#ifndef GEONKICK_BASIC_VERSION
+        layer1Button->setPressed(DspProxy::Layer::Layer1 == layer);
+        layer2Button->setPressed(DspProxy::Layer::Layer2 == layer);
+        layer3Button->setPressed(DspProxy::Layer::Layer3 == layer);
+#endif // GEONKICK_BASIC_VERSION
+        dspProxy->setLayer(layer);
         action requestUpdateGui();
 }
 
@@ -229,7 +263,7 @@ void EnvelopeWidget::updateGui()
         drawArea->update();
 }
 
-Oscillator* EnvelopeWidget::getCurrentOscillator() const
+OscillatorModel* EnvelopeWidget::getCurrentOscillator() const
 {
         if (auto envelope = drawArea->getEnvelope(); envelope)
                 return static_cast<OscillatorEnvelope*>(envelope)->getOscillator();
@@ -240,3 +274,9 @@ void EnvelopeWidget::setPointEditingMode(bool b)
 {
         drawArea->setPointEditingMode(b);
 }
+
+void EnvelopeWidget::enableBezierMode(bool b)
+{
+        GeonkickConfig().setBezierMode(b);
+        drawArea->setBezierMode(b);
+}
