@@ -1,6 +1,6 @@
 /**
  * File name: general_group_box.h
- * Project: Geonkick (A kick synthesizer)
+ * Project: Geonkick (A percussive synthesizer)
  *
  * Copyright (C) 2017 Iurie Nistor 
  *
@@ -24,30 +24,30 @@
 #ifndef GKICK_GENERAL_GROUP_BOX_H
 #define GKICK_GENERAL_GROUP_BOX_H
 
-#include "oscillator.h"
-#include "geonkick_groupbox.h"
+#include "AbstractView.h"
 
-class Oscillator;
-class GeonkickApi;
 class Knob;
 class GeonkickButton;
-class Filter;
+class PercussionModel;
+class InstrumentGlobalEffects;
 
-class GeneralGroupBox: public GeonkickGroupBox
+class GeneralGroupBox: public AbstractView
 {
  public:
-        GeneralGroupBox(GeonkickWidget *parent, GeonkickApi *api);
-        void updateGui();
+        GeneralGroupBox(GeonkickWidget *parent, PercussionModel *model);
+        void createView() override;
+        void updateView() override;
 
  protected:
+        void bindModel() override;
+        void unbindModel() override;
         void createAplitudeEnvelopeHBox();
-        void createFilterHBox();
+        void createEffects();
 
  private:
-        GeonkickApi* geonkickApi;
-        Filter *filterBox;
-        Knob *kickAmplitudeKnob;
-        Knob *kickLengthKnob;
+        Knob *instrumentAmplitudeKnob;
+        Knob *instrumentLengthKnob;
+        InstrumentGlobalEffects *globalEffects;
 };
 
 #endif //GKICK_GENERAL_GROUP_BOX_H
