@@ -1,6 +1,6 @@
 /**
  * File name: geonkick_button.cpp
- * Project: Geonkick (A kick synthesizer)
+ * Project: Geonkick (A percussive synthesizer)
  *
  * Copyright (C) 2018 Iurie Nistor
  *
@@ -39,3 +39,9 @@ void GeonkickButton::setUnpressedImage(const RkImage &img)
 {
         setImage(img, RkButton::State::Unpressed);
 }
+
+void GeonkickButton::mouseButtonPressEvent(RkMouseEvent *event)
+{
+        setFocus(true);
+        RkButton::mouseButtonPressEvent(event);
+}
