@@ -1,6 +1,6 @@
 /**
  * File name: geonkick_widget.cpp
- * Project: Geonkick (A kick synthesizer)
+ * Project: Geonkick (A percussive synthesizer)
  *
  * Copyright (C) 2018 Iurie Nistor
  *
@@ -23,7 +23,7 @@
 
 #include "geonkick_widget.h"
 
-#include <RkPainter.h>
+#include "RkPainter.h"
 
 GeonkickWidget::GeonkickWidget(RkMain& app,
                                const RkNativeWindowInfo &info,
