@@ -1,6 +1,6 @@
 /**
  * File name: geonkick_widget.h
- * Project: Geonkick (A kick synthesizer)
+ * Project: Geonkick (A percussive synthesizer)
  *
  * Copyright (C) 2018 Iurie Nistor
  *
@@ -26,8 +26,8 @@
 
 #include "globals.h"
 
-#include <RkWidget.h>
-#include <RkImage.h>
+#include "RkWidget.h"
+#include "RkImage.h"
 
 class RkPaintEvent;
 class RkNativeWindowInfo;
