@@ -1,6 +1,6 @@
 /**
  * File name: globals.h
- * Project: Geonkick (A kick synthesizer)
+ * Project: Geonkick (A percussive synthesizer)
  *
  * Copyright (C) 2017 Iurie Nistor
  *
@@ -43,40 +43,56 @@
 #include <thread>
 #include <iomanip>
 #include <math.h>
+#include <optional>
+#include <ranges>
+
+namespace fs = std::filesystem;
 
 #ifndef M_PI
     #define M_PI 3.14159265358979323846
 #endif
 
-#include <RkObject.h>
+#include "RkObject.h"
 
 #ifdef GKICK_LOG_DEBUG_LEVEL
-#define GEONKICK_LOG_INFO(msg) std::cout << "[" << GEONKICK_NAME << "] " \
-                               << " [" << std::this_thread::get_id() << "] " \
-                               << __PRETTY_FUNCTION__ << "[INFO] " \
-                               << msg << std::endl;
-#define GEONKICK_LOG_ERROR(msg) std::cout << "[" << GEONKICK_NAME << "] " \
-                                << " [" << std::this_thread::get_id() << "] " \
-                                << __PRETTY_FUNCTION__ << "[ERROR] " \
-                                << msg << std::endl;
-#define GEONKICK_LOG_DEBUG(msg) std::cout << "[" << std::this_thread::get_id() << "][" << GEONKICK_NAME << "] " \
-                                << " [" << std::this_thread::get_id() << "] " \
-                                << __PRETTY_FUNCTION__ << "[DEBUG] " \
-                                << msg << std::endl;
+#define GEONKICK_LOG_INFO(msg)                          \
+        std::cerr << "[" << GEONKICK_NAME << "] "       \
+        << "[" << std::this_thread::get_id() << "] "    \
+        << __PRETTY_FUNCTION__ << " [INFO] "            \
+        << msg << std::endl;
+
+#define GEONKICK_LOG_ERROR(msg)                         \
+        std::cerr << "[" << GEONKICK_NAME << "] "       \
+        << "[" << std::this_thread::get_id() << "] "    \
+        << __PRETTY_FUNCTION__ << " [ERROR] "           \
+        << msg << std::endl;
+
+#define GEONKICK_LOG_DEBUG(msg)                         \
+        std::cerr << "[" << GEONKICK_NAME << "] "       \
+        << "[" << std::this_thread::get_id() << "] "    \
+        << __PRETTY_FUNCTION__ << " [DEBUG] "           \
+        << msg << std::endl;
+
 #else
-#define GEONKICK_LOG_INFO(msg) std::cout << "[" << GEONKICK_NAME << "] " \
-                               << "[INFO] " << msg << std::endl;
-#define GEONKICK_LOG_ERROR(msg) std::cout << "[" << GEONKICK_NAME << "] " \
-                               << "[ERROR] " << msg << std::endl;
-#define GEONKICK_LOG_DEBUG(msg)
-#endif //  // GKICK_LIB_LOG_LEVEL_DEBUG
+#define GEONKICK_LOG_INFO(msg)                          \
+        std::cerr << "[" << GEONKICK_NAME << "] "       \
+        << "[INFO] " << msg << std::endl;
+
+#define GEONKICK_LOG_ERROR(msg)                         \
+        std::cerr << "[" << GEONKICK_NAME << "] "       \
+        << "[ERROR] " << msg << std::endl;
+
+#define GEONKICK_LOG_DEBUG(msg)   // disabled
+#endif // GKICK_LOG_DEBUG_LEVEL
+
 
 namespace Geonkick
 {
         constexpr int defaultSampleRate = GEONKICK_DEFAULT_SAMPLE_RATE;
-        constexpr char appName[] = GEONKICK_NAME;
         constexpr char applicationName[] = GEONKICK_APP_NAME;
         constexpr char applicationVersionStr[] = GEOKICK_VERSION_STRING;
+        constexpr std::string_view defaultExportFormat = "flac24";
+        constexpr int defaultBitDepth = 16;
         constexpr double toDecibel(double val)
         {
                 if (val < std::numeric_limits<double>::min())
@@ -106,6 +122,13 @@ namespace Geonkick
                 return  ss.str();
         }
 
+        inline std::string toLower(const std::string& input)
+        {
+                std::string result = input;
+                std::transform(result.begin(), result.end(), result.begin(),
+                               [](unsigned char c) { return std::tolower(c); });
+                return result;
+        }
 } // namespace Geonkick
 
 namespace GeonkickTypes
@@ -122,6 +145,7 @@ enum class Formats : int {
           Ogg    = 7,
           Sfz    = 8
 };
+
 } // namespace GeonkickTypes
 
 #endif // GEONKICK_GLOBALS_H
