@@ -1,6 +1,6 @@
 /**
  * File name: kit_state.cpp
- * Project: Geonkick (A kick synthesizer)
+ * Project: Geonkick (A percussive synthesizer)
  *
  * Copyright (C) 2020 Iurie Nistor
  *
@@ -21,7 +21,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
-#include "percussion_state.h"
+#include "InstrumentState.h"
 #include "kit_state.h"
 
 
@@ -30,6 +30,7 @@ KitState::KitState()
         , kitName{"Default"}
         , kitAuthor{"Unknown"}
 {
+        addPercussion(std::move(DspProxy::getDefaultPercussionState()));
 }
 
 bool KitState::open(const std::string &fileName)
@@ -40,9 +41,8 @@ bool KitState::open(const std::string &fileName)
         }
 
         std::filesystem::path filePath(fileName);
-        if (filePath.extension().empty()
-            || (filePath.extension() != ".gkit"
-            && filePath.extension() != ".GKIT")) {
+        auto fileExt = Geonkick::toLower(filePath.extension().string());
+        if (filePath.extension().empty() || (fileExt != ".gkit" && fileExt != ".gkick")) {
                 GEONKICK_LOG_ERROR("can't open kit. Wrong file format.");
                 return false;
         }
@@ -58,7 +58,7 @@ bool KitState::open(const std::string &fileName)
                              (std::istreambuf_iterator<char>()));
 
         sfile.close();
-        return fromJson(fileData);
+        return fromJson(fileData, fileExt == ".gkick");
 }
 
 bool KitState::save(const std::string &fileName)
@@ -70,8 +70,7 @@ bool KitState::save(const std::string &fileName)
 
         std::filesystem::path filePath(fileName);
         if (filePath.extension().empty()
-            || (filePath.extension() != ".gkit"
-             && filePath.extension() != ".GKIT")) {
+            || Geonkick::toLower(filePath.extension().string()) != ".gkit") {
                 filePath.replace_extension(".gkit");
         }
 
@@ -117,23 +116,36 @@ std::string KitState::getUrl() const
         return kitUrl;
 }
 
-const std::vector<std::unique_ptr<PercussionState>>& KitState::percussions() const
+const std::vector<std::unique_ptr<PercussionState>>& KitState::instruments() const
 {
-        return percussionsList;
+        return instrumentsList;
 }
 
-bool KitState::fromJson(const std::string &jsonData)
+ bool KitState::fromJson(const std::string &jsonData, bool oldPreset)
 {
+        instrumentsList.clear();
         rapidjson::Document document;
         document.Parse(jsonData.c_str());
         if (!document.IsObject())
                 return false;
-        return fromJsonObject(document);
+        return fromJsonObject(document, oldPreset);
 }
 
-bool KitState::fromJsonObject(const rapidjson::Value &obj)
+ bool KitState::fromJsonObject(const rapidjson::Value &obj, bool oldPreset)
 {
-        bool isOk = true;
+        // For backward compatibility
+        if (oldPreset) {
+                auto state = std::make_unique<PercussionState>();
+                if (!state->loadObject(obj))
+                        return false;
+#ifdef GEONKICK_SINGLE
+                state->setId(0);
+#endif // GEONKICK_SINGLE
+                addPercussion(std::move(state));
+                return true;
+        }
+
+        bool isOk = false;
         for (const auto &m: obj.GetObject()) {
                 if (m.name == "KitAppVersion" && m.value.IsInt())
                         kitAppVersion = m.value.GetInt();
@@ -143,19 +155,22 @@ bool KitState::fromJsonObject(const rapidjson::Value &obj)
                         setAuthor(m.value.GetString());
                 if (m.name == "url" && m.value.IsString())
                         setUrl(m.value.GetString());
-                if (m.name == "percussions" && m.value.IsArray())
-                        isOk = parsePercussions(m.value);
+                if ((m.name == "percussions" // For backward compatibility
+                     || m.name == "instruments") && m.value.IsArray()) {
+                        isOk = parseInstruments(m.value);
+                }
         }
+
         return isOk;
 }
 
-bool KitState::parsePercussions(const rapidjson::Value &percussionsArray)
+bool KitState::parseInstruments(const rapidjson::Value &instrumentsArray)
 {
-        if (percussionsArray.Empty())
+        if (instrumentsArray.Empty())
                 return false;
 
         size_t i = 0;
-        for (const auto &per: percussionsArray.GetArray()) {
+        for (const auto &per: instrumentsArray.GetArray()) {
                 auto state = std::make_unique<PercussionState>();
                 state->setId(i++);
                 if (!state->loadObject(per))
@@ -173,11 +188,11 @@ std::string KitState::toJson() const
         jsonStream << "\"name\": \"" << getName() << "\"," << std::endl;
         jsonStream << "\"author\": \"" << getAuthor() << "\"," << std::endl;
         jsonStream << "\"url\": \"" << getUrl() << "\"," << std::endl;
-        jsonStream <<  "\"percussions\": [" << std::endl;
+        jsonStream <<  "\"instruments\": [" << std::endl;
 
         size_t i = 0;
-        for (const auto &per: percussionsList) {
-                if (i < percussionsList.size() - 1)
+        for (const auto &per: instrumentsList) {
+                if (i < instrumentsList.size() - 1)
                         jsonStream << per->toJson() << "," << std::endl;
                 else
                         jsonStream << per->toJson();
@@ -188,14 +203,14 @@ std::string KitState::toJson() const
         return jsonStream.str();
 }
 
-void KitState::addPercussion(std::unique_ptr<PercussionState> percussion)
+void KitState::addPercussion(std::unique_ptr<PercussionState> instrument)
 {
-        percussionsList.push_back(std::move(percussion));
+        instrumentsList.push_back(std::move(instrument));
 }
 
 const PercussionState* KitState::getPercussion(size_t id) const
 {
-        if (id < percussionsList.size())
-                return percussionsList[id].get();
+        if (id < instrumentsList.size())
+                return instrumentsList[id].get();
         return nullptr;
 }
