@@ -1,6 +1,6 @@
 /**
  * File name: knob.cpp
- * Project: Geonkick (A kick synthesizer)
+ * Project: Geonkick (A percussive synthesizer)
  *
  * Copyright (C) 2017 Iurie Nistor 
  *
@@ -23,8 +23,8 @@
 
 #include "knob.h"
 
-#include <RkPainter.h>
-#include <RkEvent.h>
+#include "RkPainter.h"
+#include "RkEvent.h"
 
 #define GEONKICK_KNOB_MAX_DEGREE 270
 #define GEONKICK_KNOB_MIN_DEGREE 0
@@ -50,6 +50,7 @@ void Knob::setKnobImage(const RkImage &img)
 void Knob::setKnobBackgroundImage(const RkImage &img)
 {
         knobBackground = img;
+        setSize(knobBackground.size());
 }
 
 void Knob::paintWidget(RkPaintEvent *event)
