@@ -1,6 +1,6 @@
 /**
  * File name: layers_group_box.h
- * Project: Geonkick (A kick synthesizer)
+ * Project: Geonkick (A percussive synthesizer)
  *
  * Copyright (C) 2019 Iurie Nistor 
  *
@@ -27,20 +27,20 @@
 #include "geonkick_groupbox.h"
 
 class GeonkickSlider;
-class GeonkickApi;
+class DspProxy;
 class GeonkickButton;
 
 class LayersGroupBox: public GeonkickGroupBox
 {
  public:
-        LayersGroupBox(GeonkickApi *api, GeonkickWidget *parent);
+        LayersGroupBox(DspProxy *dsp, GeonkickWidget *parent);
         void updateGui();
 
  protected:
         void setLayerAmplitude(int layer, int val);
 
  private:
-        GeonkickApi *geonkickApi;
+        DspProxy *dspProxy;
         GeonkickSlider *layerSliders[3];
 };
 
