@@ -1,6 +1,6 @@
 /**
  * File name: GKickVstEditor.h
- * Project: Geonkick (A kick synthesizer)
+ * Project: Geonkick (A percussive synthesizer)
  *
  * Copyright (C) 2019 Iurie Nistor 
  *
@@ -27,6 +27,8 @@
 #include "public.sdk/source/vst/vsteditcontroller.h"
 #include "pluginterfaces/gui/iplugview.h"
 
+#include "RkEvent.h"
+
 #include <memory>
 
 using namespace Steinberg;
@@ -36,7 +38,7 @@ using namespace Linux;
 #endif // GEONKICK_OS_GNU
 
 class MainWindow;
-class GeonkickApi;
+class DspProxy;
 class RkMain;
 
 #ifdef GEONKICK_OS_GNU
@@ -60,16 +62,23 @@ class GKickVstTimer: public ITimerHandler {
 
 class GKickVstEditor : public Vst::EditorView {
  public:
-        GKickVstEditor(Vst::EditController *controller, GeonkickApi *api);
+        GKickVstEditor(Vst::EditController *controller, DspProxy *dsp);
         tresult PLUGIN_API isPlatformTypeSupported(Steinberg::FIDString type) override;
         tresult PLUGIN_API attached(void* parent, FIDString type) override;
         tresult PLUGIN_API removed() override;
         tresult PLUGIN_API getSize(ViewRect* newSize) override;
 
+#ifndef GEONKICK_OS_GNU
+ protected:
+        tresult processKey(RkEvent::Type keyType, char16 key);
+        tresult onKeyDown(char16 key, int16 keyCode, int16 modifiers) override;
+        tresult onKeyUp(char16 key, int16 keyCode, int16 modifiers) override;
+#endif // GEONKICK_OS_GNU
+
  private:
         std::unique_ptr<RkMain> guiApp;
         MainWindow *mainWindow;
-        GeonkickApi *geonkickApi;
+        DspProxy *dspProxy;
 #ifdef GEONKICK_OS_GNU
         std::unique_ptr<GKickVstTimer> loopTimer;
 #endif // GEONKICK_OS_GNU
