@@ -1,6 +1,6 @@
 /**
  * File name: GKickVstProcessor.h
- * Project: Geonkick (A percussion synthesizer)
+ * Project: Geonkick (A percussive synthesizer)
  *
  * Copyright (C) 2019 Iurie Nistor 
  *
@@ -41,7 +41,7 @@ __attribute__((visibility("default"))) bool ModuleExit (void);
 }
 #endif // __cplusplus
 
-class GeonkickApi;
+class DspProxy;
 
 using namespace Steinberg;
 
@@ -64,7 +64,8 @@ class GKickVstProcessor : public Vst::SingleComponentEffect {
         tresult PLUGIN_API setComponentState(IBStream* state) SMTG_OVERRIDE;
 
   protected:
-        std::unique_ptr<GeonkickApi> geonkickApi;
+        std::string stateData;
+        std::unique_ptr<DspProxy> dspProxy;
         std::vector<float*> channelsBuffers;
         int sampleRate;
 };
