@@ -1,8 +1,8 @@
 /**
  * File name: preset_folder.cpp
- * Project: Geonkick (A percussion synthesizer)
+ * Project: Geonkick (A percussive synthesizer)
  *
- * Copyright (C) 2020 Iurie Nistor 
+ * Copyright (C) 2020 Iurie Nistor
  *
  * This file is part of Geonkick.
  *
@@ -26,27 +26,16 @@
 
 PresetFolder::PresetFolder(const std::filesystem::path &path)
         : folderPath{path}
-        , isCustomFolder{false}
 {
         loadPresets();
 }
 
-void PresetFolder::setAsCustom(bool b)
-{
-        isCustomFolder = b;
-}
-
-bool PresetFolder::isCustom() const
-{
-        return isCustomFolder;
-}
-
 std::string PresetFolder::name() const
 {
         return folderPath.stem().string();
 }
 
-std::filesystem::path PresetFolder::path() const
+const std::filesystem::path& PresetFolder::path() const
 {
         return folderPath;
 }
@@ -61,13 +50,10 @@ bool PresetFolder::loadPresets()
         try {
                 for (const auto &entry : std::filesystem::directory_iterator(folderPath)) {
                         if (!entry.path().empty() && std::filesystem::is_regular_file(entry.path())
-                            && (entry.path().extension().string() == ".gkick"
-#ifndef GEONKICK_SINGLE
-                                || entry.path().extension().string() == ".gkit"
-#endif // GEONKICK_SINGLE
-                                )) {
-                                GEONKICK_LOG_DEBUG("preset: " << entry.path());
-                                presetList.push_back(std::make_unique<Preset>(entry.path()));
+                            && (Geonkick::toLower(entry.path().extension().string()) == ".gkit"
+                                // For backward compatibity
+                                || Geonkick::toLower(entry.path().extension().string()) == ".gkick")) {
+                                presetList.emplace_back(std::make_unique<Preset>(entry.path()));
                         }
                 }
         } catch(...) {
