@@ -34,6 +34,9 @@ class RkObject {
  public:
         explicit RkObject(RkObject *parent = nullptr);
         virtual ~RkObject();
+        RK_DECL_ACT(aboutToBeDeleted,
+                    aboutToBeDeleted(RkObject *obj),
+                    RK_ARG_TYPE(RkObject*), RK_ARG_VAL(obj));
         Rk::ObjectType type() const;
         RkObject* parent() const;
         const std::unordered_set<RkObject*>& children() const;
@@ -45,6 +48,7 @@ class RkObject {
         void setName(const std::string &name);
         std::string name() const;
         void rk__add_bound_object(RkObject* obj);
+        void unbindObject(RkObject* obj);
 
  protected:
         RK_DECLARE_O_PTR(RkObject);
