@@ -2,7 +2,7 @@
  * File name: RkEventQueueImpl.h
  * Project: Redkite (A small GUI toolkit)
  *
- * Copyright (C) 2019 Iurie Nistor 
+ * Copyright (C) 2019 Iurie Nistor
  *
  * This file is part of Redkite.
  *
@@ -29,6 +29,10 @@
 #include "RkEvent.h"
 #include "RkShortcut.h"
 
+class RkWidget;
+class RkSystemWindow;
+class RkTimer;
+
 #ifdef RK_OS_WIN
         class RkEventQueueWin;
 #elif RK_OS_MAC
@@ -39,10 +43,10 @@
 
 class RkEventQueue::RkEventQueueImpl {
  public:
-        using EventQueueWindowId = unsigned long long int;
         explicit RkEventQueueImpl(RkEventQueue* queueInterface);
         virtual ~RkEventQueueImpl();
-
+        RkSystemWindow* setTopWidget(RkWidget *widget, const RkNativeWindowInfo *parent = nullptr);
+        RkSystemWindow* getSystemWindow() const;
         bool objectExists(RkObject *t) const;
         void addObject(RkObject *obj);
         void addShortcut(RkObject *obj,
@@ -52,11 +56,8 @@ class RkEventQueue::RkEventQueueImpl {
                             Rk::Key key,
                             Rk::KeyModifiers modifier = Rk::KeyModifiers::NoModifier);
         void removeObject(RkObject *obj);
-        RkWidget* findWidget(const RkWindowId &id) const;
         void removeObjEvents(RkObject *obj);
         void postEvent(RkObject *obj, std::unique_ptr<RkEvent> event);
-        void postEvent(const RkWindowId &id, std::unique_ptr<RkEvent> event);
-        void processEvent(RkObject *obj, RkEvent *event);
         void processEvents();
         void postAction(std::unique_ptr<RkAction> act);
         void processActions();
@@ -65,14 +66,21 @@ class RkEventQueue::RkEventQueueImpl {
         void processTimers();
         void clearEvents(const RkObject *obj);
         void clearActions(const RkObject *obj);
-        RkObject* findObjectByName(const std::string &name) const;
+        void processQueue();
+        void addPopup(RkWidget* popup);
+        void removePopup(RkWidget* popup);
+        const std::vector<RkWidget*>& getPopupWidgets() const;
         void setScaleFactor(double factor);
-        double getScaleFactor() const;
+        double scaleFactor() const;
+#ifdef RK_OS_WIN
+        RkEventQueueWin* getPlatformEventQueue() const;
+#endif // RK_OS_WIN
         void dispatchEvents();
 
+
  protected:
-        void processShortcuts(RkKeyEvent *event);
         void processPopups(RkWidget *widget, RkEvent* event);
+        void processShortcuts(RkKeyEvent *event);
         void removeObjectShortcuts(RkObject *obj);
         bool isTopWidget(RkObject *obj) const;
 
@@ -81,13 +89,14 @@ class RkEventQueue::RkEventQueueImpl {
         RK_DISABLE_COPY(RkEventQueueImpl);
         RK_DISABLE_MOVE(RkEventQueueImpl);
         std::unordered_set<RkObject*> objectsList;
-        std::unordered_map<EventQueueWindowId, RkObject*> windowIdsMap;
+        std::vector<RkWidget*> popupList;
         std::unordered_map<int, std::unique_ptr<RkShortcut>> shortcutsList;
+        std::mutex eventsQueueMutex;
         std::vector<std::pair<RkObject*, std::unique_ptr<RkEvent>>> eventsQueue;
+        std::mutex actionsQueueMutex;
         std::vector<std::unique_ptr<RkAction>> actionsQueue;
         std::unordered_set<RkTimer*> timersList;
-        std::unordered_map<EventQueueWindowId, RkObject*> popupList;
-        std::mutex actionsQueueMutex;
+        std::unique_ptr<RkSystemWindow> systemWindow;
 
 #ifdef RK_OS_WIN
         std::unique_ptr<RkEventQueueWin> platformEventQueue;
