@@ -36,14 +36,9 @@ struct RkCanvasInfo;
 
 class RkWindowX {
  public:
-        explicit RkWindowX(const RkNativeWindowInfo *parent,
-                           Rk::WindowFlags flags = Rk::WindowFlags::Widget,
-                           bool isTop = false);
-        explicit RkWindowX(const RkNativeWindowInfo &parent,
-                           Rk::WindowFlags flags = Rk::WindowFlags::Widget,
-                           bool isTop = false);
+        explicit RkWindowX(const RkNativeWindowInfo *parent);
+        explicit RkWindowX(const RkNativeWindowInfo &parent);
         ~RkWindowX();
-        Rk::WindowFlags flags() const;
         bool init();
         void show(bool b);
         const RkNativeWindowInfo* nativeWindowInfo() const;
@@ -61,7 +56,7 @@ class RkWindowX {
         void setBackgroundColor(const RkColor &color);
         const RkColor& background() const;
         void resizeCanvas();
-        const RkCanvasInfo* getCanvasInfo() const;
+        RkCanvasInfo* getCanvasInfo() const;
         void update();
         void setFocus(bool b);
         bool hasFocus() const;
@@ -81,7 +76,6 @@ class RkWindowX {
         RK_DISABLE_COPY(RkWindowX);
         RK_DISABLE_MOVE(RkWindowX);
         RkNativeWindowInfo parentWindowInfo;
-        Rk::WindowFlags windowFlags;
         Display *xDisplay;
         int screenNumber;
         Window xWindow;
@@ -95,7 +89,6 @@ class RkWindowX {
         std::unique_ptr<RkNativeWindowInfo> windowInfo;
         XVisualInfo visualInfo;
         double scaleFactor;
-        bool isTopWindow;
 };
 
 #endif // RK_WIDGET_XWIN_H
